@@ -83,11 +83,6 @@ public JNIType getType() {
 	return method.getParameterTypes()[parameter];
 }
 
-@Override
-public JNIType getType64() {
-	return method.getParameterTypes64()[parameter];
-}
-
 @Override
 public int getParameter() {
 	return parameter;
