@@ -32,8 +32,7 @@ public void generate(JNIClass clazz) {
 
 public void generate(JNIMethod[] methods) {
 	sort(methods);	
-	for (int i = 0; i < methods.length; i++) {
-		JNIMethod method = methods[i];
+	for (JNIMethod method : methods) {
 		if ((method.getModifiers() & Modifier.NATIVE) == 0) continue;
 		generate(method);
 	}
