@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -22,7 +25,7 @@ import org.eclipse.swt.widgets.*;
  * individual cell in a table.
  * <p>
  * For a detailed example of using a TableCursor to navigate to a cell and then edit it see
- * http://git.eclipse.org/c/platform/eclipse.platform.swt.git/tree/examples/org.eclipse.swt.snippets/src/org/eclipse/swt/snippets/Snippet96.java .
+ * https://github.com/eclipse-platform/eclipse.platform.swt/tree/master/examples/org.eclipse.swt.snippets/src/org/eclipse/swt/snippets/Snippet96.java .
  *
  * <dl>
  * <dt><b>Styles:</b></dt>
@@ -83,82 +86,65 @@ public TableCursor(Table parent, int style) {
 	setBackground(null);
 	setForeground(null);
 
-	listener = new Listener() {
-		@Override
-		public void handleEvent(Event event) {
-			switch (event.type) {
-				case SWT.Dispose :
-					onDispose(event);
-					break;
-				case SWT.FocusIn :
-				case SWT.FocusOut :
-					redraw();
-					break;
-				case SWT.KeyDown :
-					keyDown(event);
-					break;
-				case SWT.Paint :
-					paint(event);
-					break;
-				case SWT.Traverse : {
-					event.doit = true;
-					switch (event.detail) {
-						case SWT.TRAVERSE_ARROW_NEXT :
-						case SWT.TRAVERSE_ARROW_PREVIOUS :
-						case SWT.TRAVERSE_RETURN :
-							event.doit = false;
-							break;
-					}
-					break;
+	listener = event -> {
+		switch (event.type) {
+			case SWT.Dispose :
+				onDispose(event);
+				break;
+			case SWT.FocusIn :
+			case SWT.FocusOut :
+				redraw();
+				break;
+			case SWT.KeyDown :
+				keyDown(event);
+				break;
+			case SWT.Paint :
+				paint(event);
+				break;
+			case SWT.Traverse : {
+				event.doit = true;
+				switch (event.detail) {
+					case SWT.TRAVERSE_ARROW_NEXT :
+					case SWT.TRAVERSE_ARROW_PREVIOUS :
+					case SWT.TRAVERSE_RETURN :
+						event.doit = false;
+						break;
 				}
+				break;
 			}
 		}
 	};
 	int[] events = new int[] {SWT.Dispose, SWT.FocusIn, SWT.FocusOut, SWT.KeyDown, SWT.Paint, SWT.Traverse};
-	for (int i = 0; i < events.length; i++) {
-		addListener(events[i], listener);
+	for (int event : events) {
+		addListener(event, listener);
 	}
 
-	tableListener = new Listener() {
-		@Override
-		public void handleEvent(Event event) {
-			switch (event.type) {
-				case SWT.MouseDown :
-					tableMouseDown(event);
-					break;
-				case SWT.FocusIn :
-					tableFocusIn(event);
-					break;
-			}
+	tableListener = event -> {
+		switch (event.type) {
+			case SWT.MouseDown :
+				tableMouseDown(event);
+				break;
+			case SWT.FocusIn :
+				tableFocusIn(event);
+				break;
 		}
 	};
 	table.addListener(SWT.FocusIn, tableListener);
 	table.addListener(SWT.MouseDown, tableListener);
 
-	disposeItemListener = new Listener() {
-		@Override
-		public void handleEvent(Event event) {
-			unhookRowColumnListeners();
-			row = null;
-			column = null;
-			_resize();
-		}
+	disposeItemListener = event -> {
+		unhookRowColumnListeners();
+		row = null;
+		column = null;
+		_resize();
 	};
-	disposeColumnListener = new Listener() {
-		@Override
-		public void handleEvent(Event event) {
-			unhookRowColumnListeners();
-			row = null;
-			column = null;
-			_resize();
-		}
-	};
-	resizeListener = new Listener() {
-		@Override
-		public void handleEvent(Event event) {
-			_resize();
-		}
+	disposeColumnListener = event -> {
+		unhookRowColumnListeners();
+		row = null;
+		column = null;
+		_resize();
 	};
+	resizeListener = event -> _resize();
 	ScrollBar hBar = table.getHorizontalBar();
 	if (hBar != null) {
 		hBar.addListener(SWT.Selection, resizeListener);
@@ -254,26 +240,26 @@ void keyDown(Event event) {
 		case SWT.ARROW_DOWN :
 			setRowColumn(Math.min(rowIndex + 1, table.getItemCount() - 1), columnIndex, true);
 			break;
-        case SWT.ARROW_LEFT :
-        case SWT.ARROW_RIGHT :
-        	{
-        		int columnCount = table.getColumnCount();
-        		if (columnCount == 0) break;
-        		int[] order = table.getColumnOrder();
-        		int index = 0;
-        		while (index < order.length) {
-        			if (order[index] == columnIndex) break;
-        			index++;
+		case SWT.ARROW_LEFT :
+		case SWT.ARROW_RIGHT :
+			{
+				int columnCount = table.getColumnCount();
+				if (columnCount == 0) break;
+				int[] order = table.getColumnOrder();
+				int index = 0;
+				while (index < order.length) {
+					if (order[index] == columnIndex) break;
+					index++;
 				}
 				if (index == order.length) index = 0;
-		        int leadKey = (getStyle() & SWT.RIGHT_TO_LEFT) != 0 ? SWT.ARROW_RIGHT : SWT.ARROW_LEFT;
-		        if (event.keyCode == leadKey) {
-		           setRowColumn(rowIndex, order[Math.max(0, index - 1)], true);
-		        } else {
-		           setRowColumn(rowIndex, order[Math.min(columnCount - 1, index + 1)], true);
-		        }
-		        break;
-        	}
+				int leadKey = (getStyle() & SWT.RIGHT_TO_LEFT) != 0 ? SWT.ARROW_RIGHT : SWT.ARROW_LEFT;
+				if (event.keyCode == leadKey) {
+					setRowColumn(rowIndex, order[Math.max(0, index - 1)], true);
+				} else {
+					setRowColumn(rowIndex, order[Math.min(columnCount - 1, index + 1)], true);
+				}
+				break;
+			}
 		case SWT.HOME :
 			setRowColumn(0, columnIndex, true);
 			break;
@@ -647,7 +633,7 @@ public void setSelection(int row, int column) {
 	int columnCount = table.getColumnCount();
 	int maxColumnIndex =  columnCount == 0 ? 0 : columnCount - 1;
 	if (row < 0
-	    || row >= table.getItemCount()
+		|| row >= table.getItemCount()
 		|| column < 0
 		|| column > maxColumnIndex)
 		SWT.error(SWT.ERROR_INVALID_ARGUMENT);
