@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2017 IBM Corporation and others.
+ * Copyright (c) 2000, 2019 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -15,8 +15,11 @@ package org.eclipse.swt.dnd;
 
 
 import org.eclipse.swt.*;
+import org.eclipse.swt.graphics.*;
 import org.eclipse.swt.internal.*;
 import org.eclipse.swt.internal.gtk.*;
+import org.eclipse.swt.internal.gtk3.*;
+import org.eclipse.swt.internal.gtk4.*;
 import org.eclipse.swt.widgets.*;
 
 /**
@@ -34,17 +37,17 @@ public class Clipboard {
 
 	private Display display;
 
-	static long /*int*/ GTKCLIPBOARD;
-	static long /*int*/ GTKPRIMARYCLIPBOARD;
-	private static long /*int*/ TARGET;
+	static long GTKCLIPBOARD;
+	static long GTKPRIMARYCLIPBOARD;
+	private static long TARGET;
 
 	static {
-		GTKCLIPBOARD = GTK.gtk_clipboard_get(GDK.GDK_NONE);
+		GTKCLIPBOARD = GTK.GTK4 ? GDK.gdk_display_get_clipboard(GDK.gdk_display_get_default()) : GTK3.gtk_clipboard_get (GDK.GDK_NONE);
 		byte[] buffer = Converter.wcsToMbcs("PRIMARY", true);
-		long /*int*/ primary = GDK.gdk_atom_intern(buffer, false);
-		GTKPRIMARYCLIPBOARD = GTK.gtk_clipboard_get(primary);
+		long primary = GTK.GTK4 ? 0 : GDK.gdk_atom_intern(buffer, false);
+		GTKPRIMARYCLIPBOARD = GTK.GTK4 ? GDK.gdk_display_get_primary_clipboard(GDK.gdk_display_get_default()) : GTK3.gtk_clipboard_get(primary);
 		buffer = Converter.wcsToMbcs("TARGETS", true);
-		TARGET = GDK.gdk_atom_intern(buffer, false);
+		TARGET = GTK.GTK4 ? 0 : GDK.gdk_atom_intern(buffer, false);
 	}
 
 /**
@@ -215,7 +218,7 @@ public void dispose () {
  * <p>The following snippet shows text and RTF text being retrieved from the
  * clipboard:</p>
  *
- *    <code><pre>
+ *    <pre><code>
  *    Clipboard clipboard = new Clipboard(display);
  *    TextTransfer textTransfer = TextTransfer.getInstance();
  *    String textData = (String)clipboard.getContents(textTransfer);
@@ -251,7 +254,7 @@ public Object getContents(Transfer transfer) {
  * <p>The following snippet shows text and RTF text being retrieved from the
  * clipboard:</p>
  *
- *    <code><pre>
+ *    <pre><code>
  *    Clipboard clipboard = new Clipboard(display);
  *    TextTransfer textTransfer = TextTransfer.getInstance();
  *    String textData = (String)clipboard.getContents(textTransfer);
@@ -289,42 +292,79 @@ public Object getContents(Transfer transfer) {
 public Object getContents(Transfer transfer, int clipboards) {
 	checkWidget();
 	if (transfer == null) DND.error(SWT.ERROR_NULL_ARGUMENT);
-	long /*int*/ selection_data = 0;
+
+	if(GTK.GTK4) {
+		Object result = getContents_gtk4(transfer, clipboards);
+		return result;
+	}
+
+	long selection_data = 0;
 	int[] typeIds = transfer.getTypeIds();
 	for (int i = 0; i < typeIds.length; i++) {
 		if ((clipboards & DND.CLIPBOARD) != 0) {
 			selection_data = gtk_clipboard_wait_for_contents(GTKCLIPBOARD, typeIds[i]);
-			/*
-			* This call to gdk_threads_leave() is a temporary work around
-			* to avoid deadlocks when gdk_threads_init() is called by native
-			* code outside of SWT (i.e AWT, etc). It ensures that the current
-			* thread leaves the GTK lock acquired by the function above.
-			*/
-			GDK.gdk_threads_leave();
 		}
 		if (selection_data != 0) break;
 		if ((clipboards & DND.SELECTION_CLIPBOARD) != 0) {
 			selection_data = gtk_clipboard_wait_for_contents(GTKPRIMARYCLIPBOARD, typeIds[i]);
-			/*
-			* This call to gdk_threads_leave() is a temporary work around
-			* to avoid deadlocks when gdk_threads_init() is called by native
-			* code outside of SWT (i.e AWT, etc). It ensures that the current
-			* thread leaves the GTK lock acquired by the function above.
-			*/
-			GDK.gdk_threads_leave();
 		}
 	}
 	if (selection_data == 0) return null;
 	TransferData tdata = new TransferData();
-	tdata.type = GTK.gtk_selection_data_get_data_type(selection_data);
-	tdata.pValue = GTK.gtk_selection_data_get_data(selection_data);
-	tdata.length = GTK.gtk_selection_data_get_length(selection_data);
-	tdata.format = GTK.gtk_selection_data_get_format(selection_data);
+	tdata.type = GTK3.gtk_selection_data_get_data_type(selection_data);
+	tdata.pValue = GTK3.gtk_selection_data_get_data(selection_data);
+	tdata.length = GTK3.gtk_selection_data_get_length(selection_data);
+	tdata.format = GTK3.gtk_selection_data_get_format(selection_data);
 	Object result = transfer.nativeToJava(tdata);
-	GTK.gtk_selection_data_free(selection_data);
+	GTK3.gtk_selection_data_free(selection_data);
 	return result;
 }
 
+private Object getContents_gtk4(Transfer transfer, int clipboards) {
+
+	long contents = GTK4.gdk_clipboard_get_content(Clipboard.GTKCLIPBOARD);
+	if(contents == 0) return null;
+	long value = OS.g_malloc (OS.GValue_sizeof ());
+	C.memset (value, 0, OS.GValue_sizeof ());
+
+	//Pasting of text (TextTransfer/RTFTransfer)
+	if(transfer.getTypeNames()[0].equals("text/plain") || transfer.getTypeNames()[0].equals("text/rtf")) {
+		OS.g_value_init(value, OS.G_TYPE_STRING());
+		if (!GTK4.gdk_content_provider_get_value (contents, value, null)) return null;
+		long cStr = OS.g_value_get_string(value);
+		long [] items_written = new long [1];
+		long utf16Ptr = OS.g_utf8_to_utf16(cStr, -1, null, items_written, null);
+		OS.g_free(cStr);
+		if (utf16Ptr == 0) return null;
+		int length = (int)items_written[0];
+		char[] buffer = new char[length];
+		C.memmove(buffer, utf16Ptr, length * 2);
+		OS.g_free(utf16Ptr);
+		String str = new String(buffer);
+		if(transfer.getTypeNames()[0].equals("text/rtf") && !str.contains("{\\rtf1")) {
+			return null;
+		}
+		if(transfer.getTypeNames()[0].equals("text/plain") && str.contains("{\\rtf1")){
+			return null;
+		}
+		return str;
+	}
+	if(transfer.getTypeIds()[0] == (int)GDK.GDK_TYPE_PIXBUF()) {
+		ImageData imgData = null;
+		OS.g_value_init(value, GDK.GDK_TYPE_PIXBUF());
+		if (!GTK4.gdk_content_provider_get_value (contents, value, null)) return null;
+		long pixbufObj = OS.g_value_get_object(value);
+		if (pixbufObj != 0) {
+			Image img = Image.gtk_new_from_pixbuf(Display.getCurrent(), SWT.BITMAP, pixbufObj);
+			imgData = img.getImageData();
+			img.dispose();
+		}
+		return imgData;
+	}
+	//TODO: [GTK4] Other cases
+	return null;
+}
+
 /**
  * Returns <code>true</code> if the clipboard has been disposed,
  * and <code>false</code> otherwise.
@@ -357,7 +397,7 @@ public boolean isDisposed () {
  * clipboard:
  * </p>
  *
- * <code><pre>
+ * <pre><code>
  * 	Clipboard clipboard = new Clipboard(display);
  *	String textData = "Hello World";
  *	String rtfData = "{\\rtf1\\b\\i Hello World}";
@@ -414,7 +454,7 @@ public void setContents(Object[] data, Transfer[] dataTypes) {
  * clipboard:
  * </p>
  *
- * <code><pre>
+ * <pre><code>
  * 	Clipboard clipboard = new Clipboard(display);
  *	String textData = "Hello World";
  *	String rtfData = "{\\rtf1\\b\\i Hello World}";
@@ -512,6 +552,8 @@ public TransferData[] getAvailableTypes() {
  */
 public TransferData[] getAvailableTypes(int clipboards) {
 	checkWidget();
+
+	//TODO: [GTK4] This currently will not work in GTK4
 	TransferData[] result = null;
 	if ((clipboards & DND.CLIPBOARD) != 0) {
 		int[] types = getAvailableClipboardTypes();
@@ -558,12 +600,18 @@ public TransferData[] getAvailableTypes(int clipboards) {
  */
 public String[] getAvailableTypeNames() {
 	checkWidget();
+	if(GTK.GTK4) {
+		long formatsCStr = GTK4.gdk_content_formats_to_string(GTK4.gdk_clipboard_get_formats(Clipboard.GTKCLIPBOARD));
+		String formatsStr = Converter.cCharPtrToJavaString(formatsCStr, true);
+		String[] types = formatsStr.split(" ");
+		return types;
+	}
 	int[] types1 = getAvailableClipboardTypes();
 	int[] types2 = getAvailablePrimaryTypes();
 	String[] result = new String[types1.length + types2.length];
 	int count = 0;
 	for (int i = 0; i < types1.length; i++) {
-		long /*int*/ pName = GDK.gdk_atom_name(types1[i]);
+		long pName = GDK.gdk_atom_name(types1[i]);
 		if (pName == 0) {
 			continue;
 		}
@@ -573,7 +621,7 @@ public String[] getAvailableTypeNames() {
 		result[count++] = "GTKCLIPBOARD "+new String (Converter.mbcsToWcs (buffer));
 	}
 	for (int i = 0; i < types2.length; i++) {
-		long /*int*/ pName = GDK.gdk_atom_name(types2[i]);
+		long pName = GDK.gdk_atom_name(types2[i]);
 		if (pName == 0) {
 			continue;
 		}
@@ -592,61 +640,48 @@ public String[] getAvailableTypeNames() {
 
 private  int[] getAvailablePrimaryTypes() {
 	int[] types = new int[0];
-	long /*int*/ selection_data = gtk_clipboard_wait_for_contents(GTKPRIMARYCLIPBOARD, TARGET);
-	/*
-	* This call to gdk_threads_leave() is a temporary work around
-	* to avoid deadlocks when gdk_threads_init() is called by native
-	* code outside of SWT (i.e AWT, etc). It ensures that the current
-	* thread leaves the GTK lock acquired by the function above.
-	*/
-	GDK.gdk_threads_leave();
+	long selection_data = gtk_clipboard_wait_for_contents(GTKPRIMARYCLIPBOARD, TARGET);
 	if (selection_data != 0) {
 		try {
-			int length = GTK.gtk_selection_data_get_length(selection_data);
-			int format = GTK.gtk_selection_data_get_format(selection_data);
-			long /*int*/ data = GTK.gtk_selection_data_get_data(selection_data);
+			int length = GTK3.gtk_selection_data_get_length(selection_data);
+			int format = GTK3.gtk_selection_data_get_format(selection_data);
+			long data = GTK3.gtk_selection_data_get_data(selection_data);
 			if (length != 0) {
 				types = new int[length * 8 / format];
 				C.memmove(types, data, length);
 			}
 		} finally {
-			GTK.gtk_selection_data_free(selection_data);
+			GTK3.gtk_selection_data_free(selection_data);
 		}
 	}
 	return types;
 }
 private int[] getAvailableClipboardTypes () {
+
 	int[] types = new int[0];
-	long /*int*/ selection_data  = gtk_clipboard_wait_for_contents(GTKCLIPBOARD, TARGET);
-	/*
-	* This call to gdk_threads_leave() is a temporary work around
-	* to avoid deadlocks when gdk_threads_init() is called by native
-	* code outside of SWT (i.e AWT, etc). It ensures that the current
-	* thread leaves the GTK lock acquired by the function above.
-	*/
-	GDK.gdk_threads_leave();
+	long selection_data  = gtk_clipboard_wait_for_contents(GTKCLIPBOARD, TARGET);
 	if (selection_data != 0) {
 		try {
-			int length = GTK.gtk_selection_data_get_length(selection_data);
-			int format = GTK.gtk_selection_data_get_format(selection_data);
-			long /*int*/ data = GTK.gtk_selection_data_get_data(selection_data);
+			int length = GTK3.gtk_selection_data_get_length(selection_data);
+			int format = GTK3.gtk_selection_data_get_format(selection_data);
+			long data = GTK3.gtk_selection_data_get_data(selection_data);
 			if (length != 0) {
 				types = new int[length * 8 / format];
 				C.memmove(types, data, length);
 			}
 		} finally {
-			GTK.gtk_selection_data_free(selection_data);
+			GTK3.gtk_selection_data_free(selection_data);
 		}
 	}
 	return types;
 }
 
-long /*int*/ gtk_clipboard_wait_for_contents(long /*int*/ clipboard, long /*int*/ target) {
+long gtk_clipboard_wait_for_contents(long clipboard, long target) {
 	long startTime = System.currentTimeMillis();
 	String key = "org.eclipse.swt.internal.gtk.dispatchEvent";
 	Display display = this.display;
 	display.setData(key, new int[]{GDK.GDK_PROPERTY_NOTIFY, GDK.GDK_SELECTION_CLEAR, GDK.GDK_SELECTION_REQUEST, GDK.GDK_SELECTION_NOTIFY});
-	long /*int*/ selection_data = GTK.gtk_clipboard_wait_for_contents(clipboard, target);
+	long selection_data = GTK3.gtk_clipboard_wait_for_contents(clipboard, target);
 	display.setData(key, null);
 	long duration = System.currentTimeMillis() - startTime;
 	if (selection_data == 0 && duration > 5000) {
