@@ -15,8 +15,11 @@ package org.eclipse.swt.dnd;
 
 
 import org.eclipse.swt.*;
+import org.eclipse.swt.graphics.*;
 import org.eclipse.swt.internal.*;
 import org.eclipse.swt.internal.gtk.*;
+import org.eclipse.swt.internal.gtk3.*;
+import org.eclipse.swt.internal.gtk4.*;
 import org.eclipse.swt.widgets.*;
 
 class ClipboardProxy {
@@ -28,7 +31,7 @@ class ClipboardProxy {
 	Object[] primaryClipboardData;
 	Transfer[] primaryClipboardDataTypes;
 
-	long clipboardOwner = GTK.GTK4 ? GTK.gtk_window_new() : GTK.gtk_window_new(GTK.GTK_WINDOW_TOPLEVEL);
+	long clipboardOwner = GTK.GTK4 ? GTK4.gtk_window_new() : GTK3.gtk_window_new(GTK.GTK_WINDOW_TOPLEVEL);
 
 	Display display;
 	Clipboard activeClipboard = null;
@@ -43,7 +46,7 @@ static ClipboardProxy _getInstance(final Display display) {
 	if (proxy != null) return proxy;
 	proxy = new ClipboardProxy(display);
 	display.setData(ID, proxy);
-	display.addListener(SWT.Dispose, event -> {
+	display.disposeExec(() -> {
 		ClipboardProxy clipbordProxy = (ClipboardProxy)display.getData(ID);
 		if (clipbordProxy == null) return;
 		display.setData(ID, null);
@@ -71,7 +74,7 @@ void gtk_gdk_clipboard_clear(long clipboard) {
 	if (GTK.GTK4) {
 		GDK.gdk_clipboard_set_content(clipboard, 0);
 	} else {
-		GTK.gtk_clipboard_clear(clipboard);
+		GTK3.gtk_clipboard_clear(clipboard);
 	}
 }
 
@@ -92,10 +95,10 @@ long clearFunc(long clipboard,long user_data_or_owner){
 void dispose () {
 	if (display == null) return;
 	if (activeClipboard != null) {
-		GTK.gtk_clipboard_store(Clipboard.GTKCLIPBOARD);
+		if(!GTK.GTK4) GTK3.gtk_clipboard_store(Clipboard.GTKCLIPBOARD);
 	}
 	if (activePrimaryClipboard != null) {
-		GTK.gtk_clipboard_store(Clipboard.GTKPRIMARYCLIPBOARD);
+		if(!GTK.GTK4) GTK3.gtk_clipboard_store(Clipboard.GTKPRIMARYCLIPBOARD);
 	}
 	display = null;
 	if (getFunc != null ) getFunc.dispose();
@@ -106,7 +109,13 @@ void dispose () {
 	clipboardDataTypes = null;
 	primaryClipboardData = null;
 	primaryClipboardDataTypes = null;
-	if (clipboardOwner != 0) GTK.gtk_widget_destroy (clipboardOwner);
+	if (clipboardOwner != 0) {
+		if (GTK.GTK4) {
+			GTK4.gtk_window_destroy(clipboardOwner);
+		} else {
+			GTK3.gtk_widget_destroy(clipboardOwner);
+		}
+	}
 	clipboardOwner = 0;
 }
 
@@ -116,7 +125,7 @@ void dispose () {
  */
 long getFunc(long clipboard, long selection_data, long info, long user_data_or_owner){
 	if (selection_data == 0) return 0;
-	long target = GTK.gtk_selection_data_get_target(selection_data);
+	long target = GTK3.gtk_selection_data_get_target(selection_data);
 	TransferData tdata = new TransferData();
 	tdata.type = target;
 	Transfer[] types = (clipboard == Clipboard.GTKCLIPBOARD) ? clipboardDataTypes : primaryClipboardDataTypes;
@@ -133,12 +142,15 @@ long getFunc(long clipboard, long selection_data, long info, long user_data_or_o
 	if (tdata.format < 8 || tdata.format % 8 != 0) {
 		return 0;
 	}
-	GTK.gtk_selection_data_set(selection_data, tdata.type, tdata.format, tdata.pValue, tdata.length);
+	GTK3.gtk_selection_data_set(selection_data, tdata.type, tdata.format, tdata.pValue, tdata.length);
 	OS.g_free(tdata.pValue);
 	return 1;
 }
 
 boolean setData(Clipboard owner, Object[] data, Transfer[] dataTypes, int clipboards) {
+
+	if(GTK.GTK4) return setData_gtk4(owner, data, dataTypes, clipboards);
+
 	GtkTargetEntry[] entries = new  GtkTargetEntry [0];
 	long pTargetsList = 0;
 	try {
@@ -163,7 +175,7 @@ boolean setData(Clipboard owner, Object[] data, Transfer[] dataTypes, int clipbo
 		pTargetsList = OS.g_malloc(GtkTargetEntry.sizeof * entries.length);
 		int offset = 0;
 		for (int i = 0; i < entries.length; i++) {
-			OS.memmove(pTargetsList + offset, entries[i], GtkTargetEntry.sizeof);
+			GTK3.memmove(pTargetsList + offset, entries[i], GtkTargetEntry.sizeof);
 			offset += GtkTargetEntry.sizeof;
 		}
 		if ((clipboards & DND.CLIPBOARD) != 0) {
@@ -181,10 +193,10 @@ boolean setData(Clipboard owner, Object[] data, Transfer[] dataTypes, int clipbo
 			* though we set the data again. So, this API has to be used whenever we
 			* are setting the contents.
 			*/
-			if (!GTK.gtk_clipboard_set_with_owner (Clipboard.GTKCLIPBOARD, pTargetsList, entries.length, getFuncProc, clearFuncProc, clipboardOwner)) {
+			if (!GTK3.gtk_clipboard_set_with_owner (Clipboard.GTKCLIPBOARD, pTargetsList, entries.length, getFuncProc, clearFuncProc, clipboardOwner)) {
 				return false;
 			}
-			GTK.gtk_clipboard_set_can_store(Clipboard.GTKCLIPBOARD, 0, 0);
+			GTK3.gtk_clipboard_set_can_store(Clipboard.GTKCLIPBOARD, 0, 0);
 			activeClipboard = owner;
 		}
 		if ((clipboards & DND.SELECTION_CLIPBOARD) != 0) {
@@ -192,10 +204,10 @@ boolean setData(Clipboard owner, Object[] data, Transfer[] dataTypes, int clipbo
 			primaryClipboardDataTypes = dataTypes;
 			long getFuncProc = getFunc.getAddress();
 			long clearFuncProc = clearFunc.getAddress();
-			if (!GTK.gtk_clipboard_set_with_owner (Clipboard.GTKPRIMARYCLIPBOARD, pTargetsList, entries.length, getFuncProc, clearFuncProc, clipboardOwner)) {
+			if (!GTK3.gtk_clipboard_set_with_owner (Clipboard.GTKPRIMARYCLIPBOARD, pTargetsList, entries.length, getFuncProc, clearFuncProc, clipboardOwner)) {
 				return false;
 			}
-			GTK.gtk_clipboard_set_can_store(Clipboard.GTKPRIMARYCLIPBOARD, 0, 0);
+			GTK3.gtk_clipboard_set_can_store(Clipboard.GTKPRIMARYCLIPBOARD, 0, 0);
 			activePrimaryClipboard = owner;
 		}
 		return true;
@@ -207,4 +219,58 @@ boolean setData(Clipboard owner, Object[] data, Transfer[] dataTypes, int clipbo
 		if (pTargetsList != 0) OS.g_free(pTargetsList);
 	}
 }
+
+private boolean setData_gtk4(Clipboard owner, Object[] data, Transfer[] dataTypes, int clipboards) {
+	boolean result = false;
+	long [] providers = new long[0];
+	for (int i = 0; i < dataTypes.length; i++) {
+		Transfer transfer = dataTypes[i];
+		String[] typeNames = transfer.getTypeNames();
+		//Build the GdkContentProvider for each and store in array
+		long provider = setProviderFromType(typeNames[0], data[i]);
+		if(provider != 0) {
+			long[] tmp = new long [providers.length + 1];
+			System.arraycopy(providers, 0, tmp, 0, providers.length);
+			tmp[providers.length] = provider;
+			providers = tmp;
+		}
+	}
+	//Build the GdkContentProvider Union
+	if (providers.length == 0) return false;
+	long union = GTK4.gdk_content_provider_new_union(providers, providers.length);
+
+	if ((clipboards & DND.CLIPBOARD) != 0){
+		clipboardData = data;
+		clipboardDataTypes = dataTypes;
+		result = GTK4.gdk_clipboard_set_content(Clipboard.GTKCLIPBOARD, union);
+		activeClipboard = owner;
+	}
+	return result;
+}
+
+private long setProviderFromType(String string, Object data) {
+	long provider = 0;
+
+	if (data == null ) SWT.error(SWT.ERROR_NULL_ARGUMENT);
+	else {
+		if(string.equals("text/plain") || string.equals("text/rtf")) {
+			long value = OS.g_malloc (OS.GValue_sizeof());
+			C.memset (value, 0, OS.GValue_sizeof ());
+			OS.g_value_init(value, OS.G_TYPE_STRING());
+			OS.g_value_set_string(value, Converter.javaStringToCString((String)data));
+			provider = GTK4.gdk_content_provider_new_for_value(value);
+		}
+		if(string.equals("PIXBUF")) {
+			if(!(data instanceof ImageData)) DND.error(DND.ERROR_INVALID_DATA);
+			ImageData imgData = (ImageData)data;
+			Image image = new Image(Display.getCurrent(), imgData);
+			long pixbuf = ImageList.createPixbuf(image);
+			if (pixbuf != 0) {
+				provider = GTK4.gdk_content_provider_new_typed(GDK.GDK_TYPE_PIXBUF(), pixbuf);
+			}
+			image.dispose();
+		}
+	}
+	return provider;
+}
 }
