@@ -34,11 +34,11 @@ public class RTFTransfer extends ByteArrayTransfer {
 
 	private static RTFTransfer _instance = new RTFTransfer();
 	private static final String TEXT_RTF = "text/rtf"; //$NON-NLS-1$
-	private static final int TEXT_RTF_ID = registerType(TEXT_RTF);
+	private static final int TEXT_RTF_ID = GTK.GTK4 ? 0 : registerType(TEXT_RTF);
 	private static final String TEXT_RTF2 = "TEXT/RTF"; //$NON-NLS-1$
-	private static final int TEXT_RTF2_ID = registerType(TEXT_RTF2);
+	private static final int TEXT_RTF2_ID = GTK.GTK4 ? 0 : registerType(TEXT_RTF2);
 	private static final String APPLICATION_RTF = "application/rtf"; //$NON-NLS-1$
-	private static final int APPLICATION_RTF_ID = registerType(APPLICATION_RTF);
+	private static final int APPLICATION_RTF_ID = GTK.GTK4 ? 0 : registerType(APPLICATION_RTF);
 
 private RTFTransfer() {}
 
@@ -103,11 +103,17 @@ public Object nativeToJava(TransferData transferData){
 
 @Override
 protected int[] getTypeIds() {
+	if(GTK.GTK4) {
+		return new int[] {(int) OS.G_TYPE_STRING()};
+	}
 	return new int[] {TEXT_RTF_ID, TEXT_RTF2_ID, APPLICATION_RTF_ID};
 }
 
 @Override
 protected String[] getTypeNames() {
+	if(GTK.GTK4) {
+		return new String[] {TEXT_RTF};
+	}
 	return new String[] {TEXT_RTF, TEXT_RTF2, APPLICATION_RTF};
 }
 
