@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2017 IBM Corporation and others.
+ * Copyright (c) 2000, 2019 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -10,6 +10,7 @@
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
+ *     Paul Pazderski - Bug 418714: Content copied to clipboard lost after dispose
  *******************************************************************************/
 package org.eclipse.swt.dnd;
 
@@ -220,9 +221,30 @@ public void dispose () {
 	 * The argument pDataObject is owned by the caller so reference count does not
 	 * need to be incremented.
 	 */
+	int result = COM.S_OK;
 	if (COM.OleIsCurrentClipboard(this.iDataObject.getAddress()) == COM.S_OK) {
-		COM.OleFlushClipboard();
+		result = COM.OleFlushClipboard();
 	}
+	/* Just like setContents, flushing the clipboard can fail if another application is
+	 * inspecting the clipboard at the exact moment we want to flush our content.
+	 * In this case try a few more times before accepting the failure.
+	 */
+	int retryCount = 0;
+	while (result != COM.S_OK && retryCount++ < RETRY_LIMIT) {
+		try {
+			Thread.sleep(25);
+		} catch (InterruptedException e) {
+			Thread.currentThread().interrupt();
+			break;
+		}
+		if (COM.OleIsCurrentClipboard(this.iDataObject.getAddress()) != COM.S_OK) {
+			break;
+		}
+		MSG msg = new MSG();
+		OS.PeekMessage(msg, 0, 0, 0, OS.PM_NOREMOVE | OS.PM_NOYIELD);
+		result = COM.OleFlushClipboard();
+	}
+
 	this.Release();
 	display = null;
 }
@@ -334,10 +356,8 @@ public Object getContents(Transfer transfer, int clipboards) {
 	if (result != COM.S_OK) return null;
 	IDataObject dataObject = new IDataObject(ppv[0]);
 	try {
-		TransferData[] allowed = transfer.getSupportedTypes();
-		for (int i = 0; i < allowed.length; i++) {
-			if (dataObject.QueryGetData(allowed[i].formatetc) == COM.S_OK) {
-				TransferData data = allowed[i];
+		for (TransferData data : transfer.getSupportedTypes()) {
+			if (dataObject.QueryGetData(data.formatetc) == COM.S_OK) {
 				data.pIDataObject = ppv[0];
 				return transfer.nativeToJava(data);
 			}
@@ -556,8 +576,8 @@ private int EnumFormatEtc(int dwDirection, long ppenumFormatetc) {
 	if (dwDirection == COM.DATADIR_SET) return COM.E_NOTIMPL;
 	// what types have been registered?
 	TransferData[] allowedDataTypes = new TransferData[0];
-	for (int i = 0; i < transferAgents.length; i++){
-		TransferData[] formats = transferAgents[i].getSupportedTypes();
+	for (Transfer transferAgent : transferAgents) {
+		TransferData[] formats = transferAgent.getSupportedTypes();
 		TransferData[] newAllowedDataTypes = new TransferData[allowedDataTypes.length + formats.length];
 		System.arraycopy(allowedDataTypes, 0, newAllowedDataTypes, 0, allowedDataTypes.length);
 		System.arraycopy(formats, 0, newAllowedDataTypes, allowedDataTypes.length, formats.length);
@@ -630,8 +650,8 @@ private int QueryGetData(long pFormatetc) {
 	transferData.type = transferData.formatetc.cfFormat;
 	if (transferData.type == CFSTR_PREFERREDDROPEFFECT) return COM.S_OK;
 	// is this type supported by the transfer agent?
-	for (int i = 0; i < transferAgents.length; i++){
-		if (transferAgents[i].isSupportedType(transferData))
+	for (Transfer transferAgent : transferAgents) {
+		if (transferAgent.isSupportedType(transferData))
 			return COM.S_OK;
 	}
 
