@@ -93,10 +93,10 @@ public void javaToNative(Object object, TransferData transferData) {
 	RGB[] rgbs = imgData.palette.getRGBs();
 	if (rgbs != null && colorSize > 0) {
 		int offset = BITMAPINFOHEADER.sizeof;
-		for (int j = 0; j < rgbs.length; j++) {
-			bmi[offset] = (byte)rgbs[j].blue;
-			bmi[offset + 1] = (byte)rgbs[j].green;
-			bmi[offset + 2] = (byte)rgbs[j].red;
+		for (RGB rgb : rgbs) {
+			bmi[offset] = (byte)rgb.blue;
+			bmi[offset + 1] = (byte)rgb.green;
+			bmi[offset + 2] = (byte)rgb.red;
 			bmi[offset + 3] = 0;
 			offset += 4;
 		}
