@@ -17,6 +17,7 @@ import org.eclipse.swt.*;
 import org.eclipse.swt.graphics.*;
 import org.eclipse.swt.internal.*;
 import org.eclipse.swt.internal.gtk.*;
+import org.eclipse.swt.internal.gtk3.*;
 import org.eclipse.swt.internal.opengl.glx.*;
 import org.eclipse.swt.widgets.*;
 
@@ -52,7 +53,6 @@ public class GLCanvas extends Canvas {
  */
 public GLCanvas (Composite parent, int style, GLData data) {
 	super (parent, style);
-	if (OS.IsWin32) SWT.error (SWT.ERROR_NOT_IMPLEMENTED);
 	if (data == null) SWT.error (SWT.ERROR_NULL_ARGUMENT);
 	int glxAttrib [] = new int [MAX_ATTRIBUTES];
 	int pos = 0;
@@ -109,7 +109,7 @@ public GLCanvas (Composite parent, int style, GLData data) {
 	}
 	glxAttrib [pos++] = 0;
 	GTK.gtk_widget_realize (handle);
-	long window = GTK.gtk_widget_get_window (handle);
+	long window = GTK3.gtk_widget_get_window (handle);
 
 	long xDisplay = GDK.gdk_x11_display_get_xdisplay(GDK.gdk_window_get_display(window));
 	long infoPtr = GLX.glXChooseVisual (xDisplay, OS.XDefaultScreen (xDisplay), glxAttrib);
@@ -135,7 +135,7 @@ public GLCanvas (Composite parent, int style, GLData data) {
 		GDK.GDK_EXPOSURE_MASK | GDK.GDK_POINTER_MOTION_HINT_MASK;
 	attrs.window_type = GDK.GDK_WINDOW_CHILD;
 	attrs.visual = gdkvisual;
-	glWindow = GDK.gdk_window_new (window, attrs, GDK.GDK_WA_VISUAL);
+	glWindow = GTK3.gdk_window_new (window, attrs, GDK.GDK_WA_VISUAL);
 	GDK.gdk_window_set_user_data (glWindow, handle);
 	if ((style & SWT.NO_BACKGROUND) != 0) {
 		//TODO: implement this on GTK3 as pixmaps are gone.
@@ -169,7 +169,7 @@ public GLCanvas (Composite parent, int style, GLData data) {
 			GDK.gdk_window_resize (glWindow, clientArea.width, clientArea.height);
 			break;
 		case SWT.Dispose:
-			long window1 = GTK.gtk_widget_get_window (handle);
+			long window1 = GTK3.gtk_widget_get_window (handle);
 			long xDisplay1 = gdk_x11_display_get_xdisplay (window1);
 			if (context != 0) {
 				if (GLX.glXGetCurrentContext () == context) {
@@ -201,7 +201,7 @@ public GLCanvas (Composite parent, int style, GLData data) {
  */
 public GLData getGLData () {
 	checkWidget ();
-	long window = GTK.gtk_widget_get_window (handle);
+	long window = GTK3.gtk_widget_get_window (handle);
 	long xDisplay = gdk_x11_display_get_xdisplay (window);
 	GLData data = new GLData ();
 	int [] value = new int [1];
@@ -264,7 +264,7 @@ public boolean isCurrent () {
 public void setCurrent () {
 	checkWidget ();
 	if (GLX.glXGetCurrentContext () == context) return;
-	long window = GTK.gtk_widget_get_window (handle);
+	long window = GTK3.gtk_widget_get_window (handle);
 	long xDisplay = gdk_x11_display_get_xdisplay (window);
 	GLX.glXMakeCurrent (xDisplay, xWindow, context);
 }
@@ -279,7 +279,7 @@ public void setCurrent () {
  */
 public void swapBuffers () {
 	checkWidget ();
-	long window = GTK.gtk_widget_get_window (handle);
+	long window = GTK3.gtk_widget_get_window (handle);
 	long xDisplay = gdk_x11_display_get_xdisplay (window);
 	GLX.glXSwapBuffers (xDisplay, xWindow);
 }
