@@ -23,9 +23,20 @@
 #define INC_cairo_H
 
 #include <cairo.h>
+#ifdef CAIRO_HAS_XLIB_SURFACE
 #include <cairo-xlib.h>
+#else
+#define NO_cairo_1xlib_1surface_1create
+#define NO_cairo_1xlib_1surface_1get_1height
+#define NO_cairo_1xlib_1surface_1get_1width
+#endif
 #include <string.h>
+
+#ifdef _WIN32
+#include <windows.h>
+#else
 #include <dlfcn.h>
+#endif
 
 #include "cairo_custom.h"
 
