@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2017 IBM Corporation and others.
+ * Copyright (c) 2000, 2019 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -19,7 +19,7 @@ public NSApplication() {
 	super();
 }
 
-public NSApplication(long /*int*/ id) {
+public NSApplication(long id) {
 	super(id);
 }
 
@@ -32,7 +32,7 @@ public void activateIgnoringOtherApps(boolean flag) {
 }
 
 public NSImage applicationIconImage() {
-	long /*int*/ result = OS.objc_msgSend(this.id, OS.sel_applicationIconImage);
+	long result = OS.objc_msgSend(this.id, OS.sel_applicationIconImage);
 	return result != 0 ? new NSImage(result) : null;
 }
 
@@ -40,21 +40,21 @@ public void arrangeInFront(id sender) {
 	OS.objc_msgSend(this.id, OS.sel_arrangeInFront_, sender != null ? sender.id : 0);
 }
 
-public void beginSheet(NSWindow sheet, NSWindow docWindow, id modalDelegate, long /*int*/ didEndSelector, long /*int*/ contextInfo) {
+public void beginSheet(NSWindow sheet, NSWindow docWindow, id modalDelegate, long didEndSelector, long contextInfo) {
 	OS.objc_msgSend(this.id, OS.sel_beginSheet_modalForWindow_modalDelegate_didEndSelector_contextInfo_, sheet != null ? sheet.id : 0, docWindow != null ? docWindow.id : 0, modalDelegate != null ? modalDelegate.id : 0, didEndSelector, contextInfo);
 }
 
 public NSEvent currentEvent() {
-	long /*int*/ result = OS.objc_msgSend(this.id, OS.sel_currentEvent);
+	long result = OS.objc_msgSend(this.id, OS.sel_currentEvent);
 	return result != 0 ? new NSEvent(result) : null;
 }
 
 public NSDockTile dockTile() {
-	long /*int*/ result = OS.objc_msgSend(this.id, OS.sel_dockTile);
+	long result = OS.objc_msgSend(this.id, OS.sel_dockTile);
 	return result != 0 ? new NSDockTile(result) : null;
 }
 
-public void endSheet(NSWindow sheet, long /*int*/ returnCode) {
+public void endSheet(NSWindow sheet, long returnCode) {
 	OS.objc_msgSend(this.id, OS.sel_endSheet_returnCode_, sheet != null ? sheet.id : 0, returnCode);
 }
 
@@ -79,22 +79,17 @@ public boolean isRunning() {
 }
 
 public NSWindow keyWindow() {
-	long /*int*/ result = OS.objc_msgSend(this.id, OS.sel_keyWindow);
+	long result = OS.objc_msgSend(this.id, OS.sel_keyWindow);
 	return result != 0 ? new NSWindow(result) : null;
 }
 
 public NSMenu mainMenu() {
-	long /*int*/ result = OS.objc_msgSend(this.id, OS.sel_mainMenu);
+	long result = OS.objc_msgSend(this.id, OS.sel_mainMenu);
 	return result != 0 ? new NSMenu(result) : null;
 }
 
-public NSWindow mainWindow() {
-	long /*int*/ result = OS.objc_msgSend(this.id, OS.sel_mainWindow);
-	return result != 0 ? new NSWindow(result) : null;
-}
-
-public NSEvent nextEventMatchingMask(long /*int*/ mask, NSDate expiration, NSString mode, boolean deqFlag) {
-	long /*int*/ result = OS.objc_msgSend(this.id, OS.sel_nextEventMatchingMask_untilDate_inMode_dequeue_, mask, expiration != null ? expiration.id : 0, mode != null ? mode.id : 0, deqFlag);
+public NSEvent nextEventMatchingMask(long mask, NSDate expiration, NSString mode, boolean deqFlag) {
+	long result = OS.objc_msgSend(this.id, OS.sel_nextEventMatchingMask_untilDate_inMode_dequeue_, mask, expiration != null ? expiration.id : 0, mode != null ? mode.id : 0, deqFlag);
 	return result != 0 ? new NSEvent(result) : null;
 }
 
@@ -102,28 +97,19 @@ public void orderFrontStandardAboutPanel(id sender) {
 	OS.objc_msgSend(this.id, OS.sel_orderFrontStandardAboutPanel_, sender != null ? sender.id : 0);
 }
 
-public NSArray orderedWindows() {
-	long /*int*/ result = OS.objc_msgSend(this.id, OS.sel_orderedWindows);
-	return result != 0 ? new NSArray(result) : null;
-}
-
 public void postEvent(NSEvent event, boolean flag) {
 	OS.objc_msgSend(this.id, OS.sel_postEvent_atStart_, event != null ? event.id : 0, flag);
 }
 
-public void replyToOpenOrPrint(long /*int*/ reply) {
+public void replyToOpenOrPrint(long reply) {
 	OS.objc_msgSend(this.id, OS.sel_replyToOpenOrPrint_, reply);
 }
 
-public void run() {
-	OS.objc_msgSend(this.id, OS.sel_run);
-}
-
-public long /*int*/ runModalForWindow(NSWindow theWindow) {
+public long runModalForWindow(NSWindow theWindow) {
 	return OS.objc_msgSend(this.id, OS.sel_runModalForWindow_, theWindow != null ? theWindow.id : 0);
 }
 
-public boolean sendAction(long /*int*/ theAction, id theTarget, id sender) {
+public boolean sendAction(long theAction, id theTarget, id sender) {
 	return OS.objc_msgSend_bool(this.id, OS.sel_sendAction_to_from_, theAction, theTarget != null ? theTarget.id : 0, sender != null ? sender.id : 0);
 }
 
@@ -131,7 +117,7 @@ public void sendEvent(NSEvent theEvent) {
 	OS.objc_msgSend(this.id, OS.sel_sendEvent_, theEvent != null ? theEvent.id : 0);
 }
 
-public boolean setActivationPolicy(long /*int*/ activationPolicy) {
+public boolean setActivationPolicy(long activationPolicy) {
 	return OS.objc_msgSend_bool(this.id, OS.sel_setActivationPolicy_, activationPolicy);
 }
 
@@ -156,7 +142,7 @@ public void setServicesMenu(NSMenu servicesMenu) {
 }
 
 public static NSApplication sharedApplication() {
-	long /*int*/ result = OS.objc_msgSend(OS.class_NSApplication, OS.sel_sharedApplication);
+	long result = OS.objc_msgSend(OS.class_NSApplication, OS.sel_sharedApplication);
 	return result != 0 ? new NSApplication(result) : null;
 }
 
@@ -176,13 +162,13 @@ public void unhideAllApplications(id sender) {
 	OS.objc_msgSend(this.id, OS.sel_unhideAllApplications_, sender != null ? sender.id : 0);
 }
 
-public NSWindow windowWithWindowNumber(long /*int*/ windowNum) {
-	long /*int*/ result = OS.objc_msgSend(this.id, OS.sel_windowWithWindowNumber_, windowNum);
+public NSWindow windowWithWindowNumber(long windowNum) {
+	long result = OS.objc_msgSend(this.id, OS.sel_windowWithWindowNumber_, windowNum);
 	return result != 0 ? new NSWindow(result) : null;
 }
 
 public NSArray windows() {
-	long /*int*/ result = OS.objc_msgSend(this.id, OS.sel_windows);
+	long result = OS.objc_msgSend(this.id, OS.sel_windows);
 	return result != 0 ? new NSArray(result) : null;
 }
 
