@@ -82,10 +82,6 @@ public void setTitlePosition(long titlePosition) {
 	OS.objc_msgSend(this.id, OS.sel_setTitlePosition_, titlePosition);
 }
 
-public void sizeToFit() {
-	OS.objc_msgSend(this.id, OS.sel_sizeToFit);
-}
-
 public NSCell titleCell() {
 	long result = OS.objc_msgSend(this.id, OS.sel_titleCell);
 	return result != 0 ? new NSCell(result) : null;
