@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2019 IBM Corporation and others.
+ * Copyright (c) 2000, 2020 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -95,6 +95,10 @@ public static NSEvent otherEventWithType(long type, NSPoint location, long flags
 	return result != 0 ? new NSEvent(result) : null;
 }
 
+public long phase() {
+	return OS.objc_msgSend(this.id, OS.sel_phase);
+}
+
 public static long pressedMouseButtons() {
 	return OS.objc_msgSend(OS.class_NSEvent, OS.sel_pressedMouseButtons);
 }
