@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2017 IBM Corporation and others.
+ * Copyright (c) 2000, 2019 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -19,7 +19,7 @@ public NSMutableIndexSet() {
 	super();
 }
 
-public NSMutableIndexSet(long /*int*/ id) {
+public NSMutableIndexSet(long id) {
 	super(id);
 }
 
@@ -27,17 +27,8 @@ public NSMutableIndexSet(id id) {
 	super(id);
 }
 
-public void addIndex(long /*int*/ value) {
+public void addIndex(long value) {
 	OS.objc_msgSend(this.id, OS.sel_addIndex_, value);
 }
 
-public void removeIndex(long /*int*/ value) {
-	OS.objc_msgSend(this.id, OS.sel_removeIndex_, value);
-}
-
-public static NSMutableIndexSet indexSetWithIndex(long /*int*/ value) {
-	long /*int*/ result = OS.objc_msgSend(OS.class_NSMutableIndexSet, OS.sel_indexSetWithIndex_, value);
-	return result != 0 ? new NSMutableIndexSet(result) : null;
-}
-
 }
