@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2012 IBM Corporation and others.
+ * Copyright (c) 2000, 2019 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -19,7 +19,7 @@ public NSMutableSet() {
 	super();
 }
 
-public NSMutableSet(long /*int*/ id) {
+public NSMutableSet(long id) {
 	super(id);
 }
 
@@ -31,8 +31,8 @@ public void addObjectsFromArray(NSArray array) {
 	OS.objc_msgSend(this.id, OS.sel_addObjectsFromArray_, array != null ? array.id : 0);
 }
 
-public static NSSet set() {
-	long /*int*/ result = OS.objc_msgSend(OS.class_NSMutableSet, OS.sel_set);
+public static NSMutableSet set() {
+	long result = OS.objc_msgSend(OS.class_NSMutableSet, OS.sel_set);
 	return result != 0 ? new NSMutableSet(result) : null;
 }
 
