@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2007, 2012 IBM Corporation and others.
+ * Copyright (c) 2000, 2019 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -9,15 +9,15 @@
  * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
- *     IBM Corporation - initial API and implementation
+ *    IBM Corporation - initial API and implementation
  *******************************************************************************/
-
 package org.eclipse.swt.internal.cocoa;
 
-
 public class NSSize {
-	public double /*float*/ width;
-	public double /*float*/ height;
+	/** @field cast=(CGFloat) */
+	public double width;
+	/** @field cast=(CGFloat) */
+	public double height;
 	public static final int sizeof = OS.NSSize_sizeof();
 
 	@Override
