@@ -19,7 +19,7 @@ public NSTimeZone() {
 	super();
 }
 
-public NSTimeZone(long /*int*/ id) {
+public NSTimeZone(long id) {
 	super(id);
 }
 
@@ -28,7 +28,7 @@ public NSTimeZone(id id) {
 }
 
 public static NSTimeZone defaultTimeZone() {
-	long /*int*/ result = OS.objc_msgSend(OS.class_NSTimeZone, OS.sel_defaultTimeZone);
+	long result = OS.objc_msgSend(OS.class_NSTimeZone, OS.sel_defaultTimeZone);
 	return result != 0 ? new NSTimeZone(result) : null;
 }
 
