@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2019 IBM Corporation and others.
+ * Copyright (c) 2000, 2021 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -193,10 +193,6 @@ public boolean isZoomed() {
 	return OS.objc_msgSend_bool(this.id, OS.sel_isZoomed);
 }
 
-public long level() {
-	return OS.objc_msgSend(this.id, OS.sel_level);
-}
-
 public boolean makeFirstResponder(NSResponder aResponder) {
 	return OS.objc_msgSend_bool(this.id, OS.sel_makeFirstResponder_, aResponder != null ? aResponder.id : 0);
 }
@@ -205,6 +201,12 @@ public void makeKeyAndOrderFront(id sender) {
 	OS.objc_msgSend(this.id, OS.sel_makeKeyAndOrderFront_, sender != null ? sender.id : 0);
 }
 
+public NSSize maxSize() {
+	NSSize result = new NSSize();
+	OS.objc_msgSend_stret(result, this.id, OS.sel_maxSize);
+	return result;
+}
+
 public static double minFrameWidthWithTitle(NSString aTitle, long aStyle) {
 	return OS.objc_msgSend_fpret(OS.class_NSWindow, OS.sel_minFrameWidthWithTitle_styleMask_, aTitle != null ? aTitle.id : 0, aStyle);
 }
@@ -315,6 +317,10 @@ public void setLevel(long level) {
 	OS.objc_msgSend(this.id, OS.sel_setLevel_, level);
 }
 
+public void setMaxSize(NSSize maxSize) {
+	OS.objc_msgSend(this.id, OS.sel_setMaxSize_, maxSize);
+}
+
 public void setMinSize(NSSize minSize) {
 	OS.objc_msgSend(this.id, OS.sel_setMinSize_, minSize);
 }
@@ -355,17 +361,12 @@ public void setToolbar(NSToolbar toolbar) {
 	OS.objc_msgSend(this.id, OS.sel_setToolbar_, toolbar != null ? toolbar.id : 0);
 }
 
-public NSButton standardWindowButton(long b) {
-	long result = OS.objc_msgSend(this.id, OS.sel_standardWindowButton_, b);
-	return result != 0 ? new NSButton(result) : null;
-}
-
 public long styleMask() {
 	return OS.objc_msgSend(this.id, OS.sel_styleMask);
 }
 
-public void toggleToolbarShown(id sender) {
-	OS.objc_msgSend(this.id, OS.sel_toggleToolbarShown_, sender != null ? sender.id : 0);
+public void toggleFullScreen(id sender) {
+	OS.objc_msgSend(this.id, OS.sel_toggleFullScreen_, sender != null ? sender.id : 0);
 }
 
 public NSToolbar toolbar() {
@@ -381,10 +382,6 @@ public static long windowNumberAtPoint(NSPoint point, long windowNumber) {
 	return OS.objc_msgSend(OS.class_NSWindow, OS.sel_windowNumberAtPoint_belowWindowWithWindowNumber_, point, windowNumber);
 }
 
-public long windowRef() {
-	return OS.objc_msgSend(this.id, OS.sel_windowRef);
-}
-
 public void zoom(id sender) {
 	OS.objc_msgSend(this.id, OS.sel_zoom_, sender != null ? sender.id : 0);
 }
