@@ -19,7 +19,7 @@ public WebFrame() {
 	super();
 }
 
-public WebFrame(long /*int*/ id) {
+public WebFrame(long id) {
 	super(id);
 }
 
@@ -28,16 +28,16 @@ public WebFrame(id id) {
 }
 
 public DOMDocument DOMDocument() {
-	long /*int*/ result = OS.objc_msgSend(this.id, OS.sel_DOMDocument);
+	long result = OS.objc_msgSend(this.id, OS.sel_DOMDocument);
 	return result != 0 ? new DOMDocument(result) : null;
 }
 
 public WebDataSource dataSource() {
-	long /*int*/ result = OS.objc_msgSend(this.id, OS.sel_dataSource);
+	long result = OS.objc_msgSend(this.id, OS.sel_dataSource);
 	return result != 0 ? new WebDataSource(result) : null;
 }
 
-public long /*int*/ globalContext() {
+public long globalContext() {
 	return OS.objc_msgSend(this.id, OS.sel_globalContext);
 }
 
