@@ -15,10 +15,19 @@
 #ifndef INC_gdip_H
 #define INC_gdip_H
 
+/*
+ * Windows headers will sometimes have warnings above level 2, just
+ * ignore all of them, we can't do anything about it anyway.
+ */
+#pragma warning(push, 2)
+
 #include <windows.h>
 #include <gdiplus.h>
 using namespace Gdiplus;
 
+/* Restore warnings */
+#pragma warning(pop)
+
 /* Optional custom definitions to exclude some types */
 #include "defines.h"
 
