@@ -15,7 +15,7 @@ package org.eclipse.swt.internal.ole.win32;
 
 public class IProvideClassInfo2 extends IProvideClassInfo
 {
-public IProvideClassInfo2(long /*int*/ address) {
+public IProvideClassInfo2(long address) {
 	super(address);
 }
 public int GetGUID(int dwGuidKind, GUID pGUID) {
