@@ -15,8 +15,10 @@ package org.eclipse.swt.internal.win32;
 
 public class SHFILEINFO {
 	/** @field cast=(HICON) */
-	public long /*int*/ hIcon;
+	public long hIcon;
 	public int iIcon;
 	public int dwAttributes;
-	public static int sizeof = OS.IsUnicode ? OS.SHFILEINFOW_sizeof ():  OS.SHFILEINFOA_sizeof ();
+	public char [] szDisplayName = new char [OS.MAX_PATH];
+	public char [] szTypeName = new char [80];
+	public static int sizeof = OS.SHFILEINFO_sizeof ();
 }
