@@ -68,6 +68,7 @@ public class Text extends Scrollable {
 	NSRange selectionRange;
 	id targetSearch, targetCancel;
 	long actionSearch, actionCancel;
+	APPEARANCE lastAppAppearance;
 
 	/**
 	* The maximum number of characters that can be entered
@@ -691,24 +692,6 @@ void drawBackground (long id, NSGraphicsContext context, NSRect rect) {
 		}
 	} else if ((style & SWT.MULTI) != 0) {
 		if (id != scrollView.id) return;
-
-		/*
-		 *  If background image has to be set, call fillBackground(). Else, set background color
-		 *  here directly on the NSTextView and return.
-		 */
-		if (backgroundImage == null) {
-			double [] background = this.background;
-			double alpha;
-			if (background == null) {
-				background = defaultBackground ().handle;
-				alpha = getThemeAlpha ();
-			} else {
-				alpha = background[3];
-			}
-			NSColor nsColor = NSColor.colorWithDeviceRed(background[0], background[1], background[2], alpha);
-			((NSTextView) view).setBackgroundColor(nsColor);
-			return;
-		}
 	}
 	fillBackground (view, context, rect, -1);
 }
@@ -785,10 +768,8 @@ void drawInteriorWithFrame_inView_searchfield (long id, long sel, NSRect cellFra
 
 @Override
 void drawRect(long id, long sel, NSRect rect) {
+	updateThemeColors();
 	super.drawRect(id, sel, rect);
-	if (display.appAppearance == APPEARANCE.Dark) {
-		setDefaultForeground();
-	}
 }
 
 @Override
@@ -818,7 +799,10 @@ void enableWidget(boolean enabled) {
 @Override
 Cursor findCursor () {
 	Cursor cursor = super.findCursor ();
-	return (cursor != null) ? cursor : display.getSystemCursor (SWT.CURSOR_IBEAM);
+	if (cursor == null && OS.VERSION < OS.VERSION(10, 14, 0)) {
+		cursor = display.getSystemCursor (SWT.CURSOR_IBEAM);
+	}
+	return cursor;
 }
 
 @Override
@@ -1475,13 +1459,18 @@ public void insert (String string) {
 }
 
 void insertEditText (String string) {
+	_insertEditText (string, false);
+}
+
+void _insertEditText (String string, boolean enableUndo) {
 	int length = string.length ();
 	Point selection = getSelection ();
 	if (hasFocus () && hiddenText == null) {
 		if (textLimit != LIMIT) {
 			int charCount = getCharCount();
-			if (charCount - (selection.y - selection.x) + length > textLimit) {
-				length = textLimit - charCount + (selection.y - selection.x);
+			int selectionLength = selection.y - selection.x;
+			if (charCount - selectionLength + length > textLimit) {
+				length = textLimit - charCount + selectionLength;
 				length = Math.max(0, length);
 			}
 		}
@@ -1489,7 +1478,17 @@ void insertEditText (String string) {
 		string.getChars (0, buffer.length, buffer, 0);
 		NSString nsstring = NSString.stringWithCharacters (buffer, buffer.length);
 		NSText fieldEditor = ((NSTextField) view).currentEditor ();
-		if (fieldEditor != null) fieldEditor.replaceCharactersInRange (fieldEditor.selectedRange (), nsstring);
+		if (fieldEditor != null) {
+			if (enableUndo) {
+				NSUndoManager undoManager = view.undoManager();
+				if (undoManager == null) undoManager = fieldEditor.undoManager();
+				if (undoManager != null) undoManager.beginUndoGrouping ();
+				OS.objc_msgSend (fieldEditor.id, OS.sel_insertText_replacementRange_, nsstring.id, fieldEditor.selectedRange ());
+				if (undoManager != null) undoManager.endUndoGrouping ();
+			} else {
+				fieldEditor.replaceCharactersInRange (fieldEditor.selectedRange (), nsstring);
+			}
+		}
 		selectionRange = null;
 	} else {
 		String oldText = getText ();
@@ -1540,6 +1539,10 @@ boolean isNeeded(ScrollBar scrollbar) {
  */
 public void paste () {
 	checkWidget ();
+	_paste (false);
+}
+
+void _paste (boolean enableUndo) {
 	if ((style & SWT.READ_ONLY) != 0) return;
 	boolean paste = true;
 	String oldText = null;
@@ -1551,7 +1554,7 @@ public void paste () {
 			if (newText == null) return;
 			if (!newText.equals (oldText)) {
 				if ((style & SWT.SINGLE) != 0) {
-					insertEditText (newText);
+					_insertEditText (newText, enableUndo);
 				} else {
 					NSTextView textView = (NSTextView) view;
 					NSRange range = textView.selectedRange ();
@@ -1571,7 +1574,7 @@ public void paste () {
 		if ((style & SWT.SINGLE) != 0) {
 			if (oldText == null) oldText = getClipboardText ();
 			if (oldText == null) return;
-			insertEditText (oldText);
+			_insertEditText (oldText, enableUndo);
 		} else {
 			if (textLimit != LIMIT) {
 				if (oldText == null) oldText = getClipboardText ();
@@ -1738,7 +1741,7 @@ boolean sendKeyEvent (NSEvent nsEvent, int type) {
 				if ((style & SWT.PASSWORD) == 0) copy ();
 				return false;
 			case 9: /* V */
-				paste ();
+				_paste (true);
 				return false;
 			case 0: /* A */
 				NSApplication.sharedApplication().sendAction(OS.sel_selectAll_, null, NSApplication.sharedApplication());
@@ -1813,7 +1816,19 @@ void sendCancelSelection () {
 @Override
 void setBackgroundColor(NSColor nsColor) {
 	if ((style & SWT.SINGLE) != 0) {
-		((NSTextField) view).setBackgroundColor (nsColor);
+		NSTextField textField = (NSTextField) view;
+		textField.setBackgroundColor (nsColor);
+		/*
+		 * When the Text widget has focus, the background color is not drawn,
+		 * as the field editor is drawn on top. Set drawsBackground to true
+		 * for the field editor.
+		 */
+		if ((style & SWT.SEARCH) == 0 && hasFocus()) {
+			NSText fieldEditor = textField.currentEditor();
+			if (fieldEditor != null) {
+				fieldEditor.setDrawsBackground(true);
+			}
+		}
 	} else {
 		((NSTextView) view).setBackgroundColor (nsColor);
 	}
@@ -1827,16 +1842,10 @@ void setBackgroundImage(NSImage image) {
 	} else {
 		((NSTextView) view).setDrawsBackground(image == null);
 		scrollView.setDrawsBackground(image == null);
-	}
-}
 
-void setDefaultForeground() {
-	if ((style & SWT.MULTI) != 0) {
-		if (foreground != null) return;
-		if (getEnabled ()) {
-			((NSTextView) view).setTextColor (NSColor.textColor ());
-		} else {
-			((NSTextView) view).setTextColor (NSColor.disabledControlTextColor ());
+		if (image == null) {
+			// Recalculate theme colors lazily
+			lastAppAppearance = null;
 		}
 	}
 }
@@ -2485,6 +2494,31 @@ void updateCursorRects (boolean enabled) {
 	contentView.setDocumentCursor (enabled ? NSCursor.IBeamCursor () : null);
 }
 
+void updateThemeColors() {
+	// See code comment in Link.updateThemeColors() for explanation
+
+	// Avoid infinite loop of redraws
+	if (lastAppAppearance == display.appAppearance) return;
+	lastAppAppearance = display.appAppearance;
+	// Only multi-line controls are affected
+	if ((style & SWT.MULTI) == 0) return;
+
+	if (foreground == null) {
+		if (getEnabled ()) {
+			((NSTextView) view).setTextColor (NSColor.textColor ());
+		} else {
+			((NSTextView) view).setTextColor (NSColor.disabledControlTextColor ());
+		}
+	}
+
+	if ((backgroundImage == null) && (background == null)) {
+		double [] background = defaultBackground ().handle;
+		double alpha = getThemeAlpha ();
+		NSColor nsColor = NSColor.colorWithDeviceRed(background[0], background[1], background[2], alpha);
+		((NSTextView) view).setBackgroundColor(nsColor);
+	}
+}
+
 String verifyText (String string, int start, int end, NSEvent keyEvent) {
 	Event event = new Event ();
 	if (keyEvent != null) setKeyState(event, SWT.MouseDown, keyEvent);
