@@ -586,6 +586,14 @@ int [] layoutUnified (int width, int height, boolean resize) {
 	for (int i=0; i<itemCount; i++) {
 		Point size = sizes [i] = items [i].computeSize ();
 		containerRects [i] = new NSView(itemViewers.objectAtIndex(i)).frame();
+		// workaround for https://bugs.eclipse.org/bugs/show_bug.cgi?id=539693
+		if (containerRects [i].width < 0 || containerRects [i].width > 100000) {
+			containerRects [i].x = 0;
+			containerRects [i].y = 0;
+			containerRects [i].width = 0;
+			containerRects [i].height = 0;
+		}
+
 		if (i == 0) x = (int) containerRects[0].x;
 		itemHeight = Math.max (itemHeight, size.y);
 	}
@@ -766,8 +774,15 @@ void setBounds (int x, int y, int width, int height, boolean move, boolean resiz
 @Override
 void setFont(NSFont font) {
 	for (int i = 0; i < itemCount; i++) {
-		ToolItem item = items[i];
-		if (item.button != null) ((NSButton)item.button).setAttributedTitle(item.createString());
+		items[i].updateStyle ();
+	}
+}
+
+@Override
+void setForeground (double [] color) {
+	super.setForeground (color);
+	for (int i = 0; i < itemCount; i++) {
+		items[i].updateStyle ();
 	}
 }
 
