@@ -35,7 +35,7 @@ public class Callback {
 	Object object;
 	String method, signature;
 	int argCount;
-	long /*int*/ address, errorResult;
+	long address, errorResult;
 	boolean isStatic, isArrayBased;
 
 	static final boolean is32Bit = C.PTR_SIZEOF == 4 ? true : false;
@@ -108,7 +108,7 @@ public Callback (Object object, String method, int argCount, boolean isArrayBase
  * @param isArrayBased <code>true</code> if the arguments should be passed in an array and false otherwise
  * @param errorResult the return value if the java code throws an exception
  */
-public Callback (Object object, String method, int argCount, boolean isArrayBased, long /*int*/ errorResult) {
+public Callback (Object object, String method, int argCount, boolean isArrayBased, long errorResult) {
 
 	/* Set the callback fields */
 	this.object = object;
@@ -135,6 +135,7 @@ public Callback (Object object, String method, int argCount, boolean isArrayBase
 
 	/* Bind the address */
 	address = bind (this, object, method, signature, argCount, isStatic, isArrayBased, errorResult);
+	if (address == 0) SWT.error (SWT.ERROR_NO_MORE_CALLBACKS);
 }
 
 
@@ -215,11 +216,13 @@ public Callback (Object object, String method, Type returnType, Type [] argument
 	};
 
 	StringBuilder signature = new StringBuilder("(");
-	for (Type t : arguments) {
-		if (t.equals(void.class)) {
-			SWT.error(SWT.ERROR_INVALID_ARGUMENT, null, "void is not a valid argument");
+	if (this.argCount > 0) {
+		for (Type t : arguments) {
+			if (t.equals(void.class)) {
+				SWT.error(SWT.ERROR_INVALID_ARGUMENT, null, "void is not a valid argument");
+			}
+			signature.append(getTypeLetter.apply(t));
 		}
-		signature.append(getTypeLetter.apply(t));
 	}
 	signature.append(")");
 	signature.append(getTypeLetter.apply(returnType));
@@ -230,10 +233,9 @@ public Callback (Object object, String method, Type returnType, Type [] argument
 
 	/* Bind the address */
 	address = bind (this, this.object, this.method, this.signature, this.argCount, this.isStatic, this.isArrayBased, this.errorResult);
+	if (address == 0) SWT.error (SWT.ERROR_NO_MORE_CALLBACKS);
 }
 
-
-
 /**
  * Allocates the native level resources associated with the
  * callback. This method is only invoked from within the
@@ -248,7 +250,7 @@ public Callback (Object object, String method, Type returnType, Type [] argument
  * @param isArrayBased whether the callback's method is array based
  * @param errorResult the callback's error result
  */
-static native synchronized long /*int*/ bind (Callback callback, Object object, String method, String signature, int argCount, boolean isStatic, boolean isArrayBased, long /*int*/ errorResult);
+static native synchronized long bind (Callback callback, Object object, String method, String signature, int argCount, boolean isStatic, boolean isArrayBased, long errorResult);
 
 /**
  * Releases the native level resources associated with the callback,
@@ -269,7 +271,7 @@ public void dispose () {
  *
  * @return the callback address
  */
-public long /*int*/ getAddress () {
+public long getAddress () {
 	return address;
 }
 
