@@ -171,34 +171,28 @@ long gtk_commit (long imcontext, long text) {
 @Override
 long gtk_draw (long widget, long cairo) {
 	if ((state & OBSCURED) != 0) return 0;
-	long result;
-	if ( GTK.GTK_VERSION < OS.VERSION(3, 22, 0)) {
-		boolean isFocus = caret != null && caret.isFocusCaret ();
-		if (isFocus) caret.killFocus ();
-		result = super.gtk_draw (widget, cairo);
-		if (isFocus) caret.setFocus ();
-	} else {
-		result = super.gtk_draw (widget, cairo);
-		/*
-		 *  blink is needed to be checked as gtk_draw() signals sent from other parts of the canvas
-		 *  can interfere with the blinking state. This will ensure that we are only draw/redrawing the
-		 *  caret when it is intended to. See Bug 517487.
-		 *
-		 *  Additionally, only draw the caret if it has focus. See bug 528819.
-		 */
-		if (caret != null && blink == true && caret.isFocusCaret()) {
-			drawCaret(widget,cairo);
-			blink = false;
-		}
+	long result = super.gtk_draw (widget, cairo);
+	/*
+	 *  blink is needed to be checked as gtk_draw() signals sent from other parts of the canvas
+	 *  can interfere with the blinking state. This will ensure that we are only draw/redrawing the
+	 *  caret when it is intended to. See Bug 517487.
+	 *
+	 *  Additionally, only draw the caret if it has focus. See bug 528819.
+	 */
+	if (caret != null && blink == true && caret.isFocusCaret()) {
+		drawCaret(widget,cairo);
+		blink = false;
 	}
 	return result;
 }
 
-private void drawCaret (long widget, long cairo) {
+private void drawCaret(long widget, long cairo) {
 	if(this.isDisposed()) return;
 	if (cairo == 0) error(SWT.ERROR_NO_HANDLES);
+
 	if (drawFlag) {
 		Cairo.cairo_save(cairo);
+
 		if (caret.image != null && !caret.image.isDisposed() && caret.image.mask == 0) {
 			Cairo.cairo_set_source_rgb(cairo, 1, 1, 1);
 			Cairo.cairo_set_operator(cairo, Cairo.CAIRO_OPERATOR_DIFFERENCE);
@@ -225,14 +219,14 @@ private void drawCaret (long widget, long cairo) {
 			int nX = caret.x;
 			if ((style & SWT.MIRRORED) != 0) nX = getClientWidth () - nWidth - nX;
 			Cairo.cairo_rectangle(cairo, nX, caret.y, nWidth, nHeight);
-			}
+		}
+
 		Cairo.cairo_fill(cairo);
 		Cairo.cairo_restore(cairo);
 		drawFlag = false;
 	} else {
 		drawFlag = true;
-		}
-	return;
+	}
 }
 
 @Override
@@ -345,25 +339,23 @@ void scrollInPixels (int destX, int destY, int x, int y, int width, int height,
 	 * location the scrollbars are re-painted when scrolling, causing the
 	 * hopping effect. See bug 480458.
 	 */
-	if (GTK.GTK_VERSION >= OS.VERSION(3, 16, 0)) {
-		long hBarHandle = 0;
-		long vBarHandle = 0;
-		if (GTK.GTK_IS_SCROLLED_WINDOW(scrolledHandle)) {
-			hBarHandle = GTK.gtk_scrolled_window_get_hscrollbar (scrolledHandle);
-			vBarHandle = GTK.gtk_scrolled_window_get_vscrollbar (scrolledHandle);
+	long hBarHandle = 0;
+	long vBarHandle = 0;
+	if (GTK.GTK_IS_SCROLLED_WINDOW(scrolledHandle)) {
+		hBarHandle = GTK.gtk_scrolled_window_get_hscrollbar (scrolledHandle);
+		vBarHandle = GTK.gtk_scrolled_window_get_vscrollbar (scrolledHandle);
+	}
+	GtkRequisition requisition = new GtkRequisition();
+	if (hBarHandle != 0) {
+		gtk_widget_get_preferred_size (hBarHandle, requisition);
+		if (requisition.height > 0) {
+			srcRect.y = y - requisition.height;
 		}
-		GtkRequisition requisition = new GtkRequisition();
-		if (hBarHandle != 0) {
-			gtk_widget_get_preferred_size (hBarHandle, requisition);
-			if (requisition.height > 0) {
-				srcRect.y = y - requisition.height;
-			}
-		}
-		if (vBarHandle != 0) {
-			gtk_widget_get_preferred_size (vBarHandle, requisition);
-			if (requisition.width > 0) {
-				srcRect.x = x - requisition.width;
-			}
+	}
+	if (vBarHandle != 0) {
+		gtk_widget_get_preferred_size (vBarHandle, requisition);
+		if (requisition.width > 0) {
+			srcRect.x = x - requisition.width;
 		}
 	}
 	srcRect.width = width;
@@ -449,9 +441,7 @@ void scrollInPixels (int destX, int destY, int x, int y, int width, int height,
 	 * Due to overlay drawing of scrollbars current method of scrolling leaves scrollbar and notifiers for them inside the canvas
 	 * after scroll. Fix is to redraw once done.
 	 */
-	if (GTK.GTK_VERSION >= OS.VERSION(3, 16, 0)) {
-		redraw(false);
-	}
+	redraw(false);
 }
 
 @Override
