@@ -21,6 +21,7 @@ import org.eclipse.swt.internal.*;
 import org.eclipse.swt.internal.gtk.*;
 import org.eclipse.swt.internal.gtk3.*;
 import org.eclipse.swt.internal.gtk4.*;
+import org.eclipse.swt.widgets.Menu.*;
 
 /**
  * Instances of this class represent a selectable user interface object
@@ -61,9 +62,12 @@ public class MenuItem extends Item {
 	int accelerator, userId;
 	String toolTipText;
 
+	Image defaultDisableImage;
+	boolean enabled = true;
+
 	/** GTK4 only fields */
 	long modelHandle, actionHandle, shortcutHandle;
-	int parentMenuPosition;
+	Section section;
 	String actionName;
 
 /**
@@ -261,61 +265,90 @@ protected void checkSubclass () {
 }
 
 @Override
-void createHandle (int index) {
+void createHandle(int index) {
 	state |= HANDLE;
-	byte [] buffer = new byte [1];
 	int bits = SWT.CHECK | SWT.RADIO | SWT.PUSH | SWT.SEPARATOR | SWT.CASCADE;
 
 	if (GTK.GTK4) {
-		modelHandle = OS.g_menu_new();
-
 		switch (style & bits) {
 			case SWT.SEPARATOR:
+				modelHandle = OS.g_menu_new();
 				handle = OS.g_menu_item_new_section(null, modelHandle);
-				parent.sectionModelHandle = modelHandle;
 				break;
 			case SWT.RADIO:
-				Menu popoverMenuWidget = parent;
-				while (popoverMenuWidget.actionGroup == 0) {
-					popoverMenuWidget = popoverMenuWidget.cascade.parent;
-				}
-
 				long stringVariantType = OS.g_variant_type_new(OS.G_VARIANT_TYPE_STRING);
 				actionHandle = OS.g_simple_action_new_stateful(
-						Converter.javaStringToCString(String.valueOf(this.hashCode())),
-						stringVariantType,
-						OS.g_variant_new_string(Converter.javaStringToCString("untoggled")));
-				OS.g_action_map_add_action(popoverMenuWidget.actionGroup, actionHandle);
-				actionName = String.valueOf(popoverMenuWidget.hashCode()) + "." + String.valueOf(this.hashCode()) + "::toggled";
+					Converter.javaStringToCString(String.valueOf(this.hashCode())),
+					stringVariantType,
+					OS.g_variant_new_string(Converter.javaStringToCString("untoggled")));
+				OS.g_action_map_add_action(parent.actionGroup, actionHandle);
+				actionName = String.valueOf(parent.hashCode()) + "." + String.valueOf(this.hashCode()) + "::toggled";
 				handle = OS.g_menu_item_new(null, Converter.javaStringToCString(actionName));
 				OS.g_variant_type_free(stringVariantType);
 				break;
+			case SWT.CHECK:
+				long boolVariantType = OS.g_variant_type_new(OS.G_VARIANT_TYPE_BOOLEAN);
+				actionHandle = OS.g_simple_action_new_stateful(
+					Converter.javaStringToCString(String.valueOf(this.hashCode())),
+					0,
+					OS.g_variant_new_boolean(false));
+				OS.g_action_map_add_action(parent.actionGroup, actionHandle);
+				actionName = String.valueOf(parent.hashCode()) + "." + String.valueOf(this.hashCode());
+				handle = OS.g_menu_item_new(null, Converter.javaStringToCString(actionName));
+				OS.g_variant_type_free(boolVariantType);
+				break;
 			case SWT.CASCADE:
-				handle = OS.g_menu_item_new_submenu(null, modelHandle);
+				modelHandle = OS.g_menu_new();
+				handle = OS.g_menu_item_new_submenu(Converter.javaStringToCString(""), modelHandle);
 				break;
 			case SWT.PUSH:
 			default:
-				popoverMenuWidget = parent;
-				while (popoverMenuWidget.actionGroup == 0) {
-					popoverMenuWidget = popoverMenuWidget.cascade.parent;
-				}
-
 				actionHandle = OS.g_simple_action_new(Converter.javaStringToCString(String.valueOf(this.hashCode())), 0);
-				OS.g_action_map_add_action(popoverMenuWidget.actionGroup, actionHandle);
-				actionName = String.valueOf(popoverMenuWidget.hashCode()) + "." + String.valueOf(this.hashCode());
+				OS.g_action_map_add_action(parent.actionGroup, actionHandle);
+				actionName = String.valueOf(parent.hashCode()) + "." + String.valueOf(this.hashCode());
 				handle = OS.g_menu_item_new(null, Converter.javaStringToCString(actionName));
 				break;
 		}
 
-		if ((style & SWT.SEPARATOR) != 0) {
-			OS.g_menu_insert_item(parent.modelHandle, index, handle);
-		} else {
-			OS.g_menu_insert_item(parent.sectionModelHandle, index, handle);
+
+		Section selectedSection = parent.sections.getLast();
+		for (Section section : parent.sections) {
+			int sectionPosition = section.getSectionPosition();
+			int sectionLength = section.getSectionSize();
+
+			if (index > sectionPosition && index <= sectionPosition + sectionLength + 1) {
+				selectedSection = section;
+				break;
+			}
 		}
 
-		parentMenuPosition = index;
-		parent.items.add(this);
+		int sectionRelativeIndex = index - (selectedSection.getSectionPosition() + 1);
+		if ((style & SWT.SEPARATOR) != 0) {
+			section = parent.new Section(this);
+
+			int itemsToMove = selectedSection.sectionItems.size() - sectionRelativeIndex;
+			for (int i = 0; i < itemsToMove; i++) {
+				MenuItem removedItem = selectedSection.sectionItems.remove(sectionRelativeIndex);
+				section.sectionItems.add(removedItem);
+
+				OS.g_menu_remove(selectedSection.getSectionHandle(), sectionRelativeIndex);
+				OS.g_menu_insert_item(modelHandle, section.sectionItems.indexOf(removedItem), removedItem.handle);
+				removedItem.section = section;
+			}
+
+			int sectionInsertIndex = parent.sections.indexOf(selectedSection) + 1;
+			parent.sections.add(sectionInsertIndex, section);
+			OS.g_menu_insert_item(parent.modelHandle, sectionInsertIndex, handle);
+		} else {
+			section = selectedSection;
+			selectedSection.sectionItems.add(sectionRelativeIndex, this);
+			OS.g_menu_insert_item(selectedSection.getSectionHandle(), sectionRelativeIndex, handle);
+		}
+
+		parent.items.add(index, this);
 	} else {
+		byte[] buffer = new byte[1];
+
 		switch (style & bits) {
 			case SWT.SEPARATOR:
 				handle = GTK3.gtk_separator_menu_item_new ();
@@ -403,37 +436,35 @@ void createHandle (int index) {
 				}
 				break;
 		}
+
 		if (imageHandle != 0) {
 			if (OS.SWT_PADDED_MENU_ITEMS) {
 				GTK.gtk_image_set_pixel_size (imageHandle, 16);
 			}
-			if (GTK.GTK4) {
-				GTK4.gtk_box_append(boxHandle, imageHandle);
-			} else {
-				GTK3.gtk_container_add (boxHandle, imageHandle);
-				GTK.gtk_widget_show (imageHandle);
-			}
+
+			GTK3.gtk_container_add (boxHandle, imageHandle);
+			GTK.gtk_widget_show (imageHandle);
 		}
+
 		if (labelHandle != 0) {
 			GTK.gtk_label_set_xalign (labelHandle, 0);
 			GTK.gtk_widget_set_halign (labelHandle, GTK.GTK_ALIGN_FILL);
 			gtk_box_pack_end (boxHandle, labelHandle, true, true, 0);
 			GTK.gtk_widget_show (labelHandle);
 		}
+
 		if (boxHandle != 0) {
-			if (GTK.GTK4) {
-				// TODO: need to implement how menu items will be populated, currently no value for handle
-			} else {
-				GTK3.gtk_container_add (handle, boxHandle);
-				GTK.gtk_widget_show (boxHandle);
-			}
+			GTK3.gtk_container_add (handle, boxHandle);
+			GTK.gtk_widget_show (boxHandle);
 		}
+
 		if ((style & SWT.SEPARATOR) == 0) {
 			if (boxHandle == 0) {
 				labelHandle = GTK3.gtk_bin_get_child (handle);
 			}
 			GTK3.gtk_accel_label_set_accel_widget (labelHandle, 0);
 		}
+
 		long parentHandle = parent.handle;
 		boolean enabled = GTK.gtk_widget_get_sensitive (parentHandle);
 		if (!enabled) GTK.gtk_widget_set_sensitive (parentHandle, true);
@@ -477,20 +508,6 @@ long getAccelGroup () {
 	return shell.menuBar == menu ? shell.accelGroup : 0;
 }
 
-/*public*/ Rectangle getBounds () {
-	checkWidget();
-	if (!GTK.gtk_widget_get_mapped (handle)) {
-		return new Rectangle (0, 0, 0, 0);
-	}
-	GtkAllocation allocation = new GtkAllocation ();
-	GTK.gtk_widget_get_allocation (handle, allocation);
-	int x = allocation.x;
-	int y = allocation.y;
-	int width = allocation.width;
-	int height = allocation.height;
-	return new Rectangle (x, y, width, height);
-}
-
 /**
  * Returns <code>true</code> if the receiver is enabled, and
  * <code>false</code> otherwise. A disabled menu item is typically
@@ -510,6 +527,10 @@ public boolean getEnabled () {
 	checkWidget();
 
 	if (GTK.GTK4) {
+		if ((style & SWT.CASCADE) != 0) {
+			return true;
+		}
+
 		return OS.g_action_get_enabled(actionHandle);
 	} else {
 		return GTK.gtk_widget_get_sensitive(handle);
@@ -593,8 +614,12 @@ public boolean getSelection () {
 
 	if (GTK.GTK4) {
 		long gVariantState = OS.g_action_get_state(actionHandle);
-		String stateString = Converter.cCharPtrToJavaString(OS.g_variant_get_string(gVariantState, null), false);
-		return stateString.equals("toggled");
+		if ((style & SWT.CHECK) != 0) {
+			return OS.g_variant_get_boolean(gVariantState);
+		} else {
+			String stateString = Converter.cCharPtrToJavaString(OS.g_variant_get_string(gVariantState, null), false);
+			return stateString.equals("toggled");
+		}
 	} else {
 		return GTK3.gtk_check_menu_item_get_active(handle);
 	}
@@ -650,7 +675,7 @@ long gtk_select (long item) {
 }
 
 @Override
-long gtk_show_help (long widget, long helpType) {
+long gtk3_show_help (long widget, long helpType) {
 	boolean handled = hooks (SWT.Help);
 	if (handled) {
 		postEvent (SWT.Help);
@@ -658,7 +683,6 @@ long gtk_show_help (long widget, long helpType) {
 		handled = parent.sendHelpEvent (helpType);
 	}
 	if (handled) {
-		// TODO: GTK4 there is no idea of a menu shell
 		GTK3.gtk_menu_shell_deactivate (parent.handle);
 		return 1;
 	}
@@ -666,18 +690,17 @@ long gtk_show_help (long widget, long helpType) {
 }
 
 @Override
-void hookEvents () {
-	super.hookEvents ();
+void hookEvents() {
+	super.hookEvents();
 
 	if (GTK.GTK4) {
-		// Bind activate signal only for menu items with actions (SWT.CHECK and SWT.RADIO)
-		if (actionHandle != 0) {
+		if ((style & SWT.PUSH) != 0 || (style & SWT.RADIO) != 0) {
 			OS.g_signal_connect(actionHandle, OS.activate, display.activateProc, handle);
 		}
 	} else {
 		OS.g_signal_connect_closure(handle, OS.activate, display.getClosure (ACTIVATE), false);
-		OS.g_signal_connect_closure (handle, OS.select, display.getClosure (SELECT), false);
-		OS.g_signal_connect_closure_by_id (handle, display.signalIds [SHOW_HELP], 0, display.getClosure (SHOW_HELP), false);
+		OS.g_signal_connect_closure(handle, OS.select, display.getClosure (SELECT), false);
+		OS.g_signal_connect_closure_by_id(handle, display.signalIds[SHOW_HELP], 0, display.getClosure(SHOW_HELP), false);
 	}
 }
 
@@ -710,24 +733,66 @@ void releaseChildren (boolean destroy) {
 }
 
 @Override
-void releaseParent () {
-	super.releaseParent ();
+void releaseParent() {
+	super.releaseParent();
+
 	if (menu != null) {
 		if (menu.selectedItem == this) menu.selectedItem = null;
-		menu.dispose ();
+		menu.dispose();
+
+		menu = null;
 	}
-	menu = null;
 }
 
 @Override
-void releaseWidget () {
-	super.releaseWidget ();
-	long accelGroup = getAccelGroup ();
-	if (accelGroup != 0) removeAccelerator (accelGroup);
-	if (groupHandle != 0) OS.g_object_unref (groupHandle);
-	groupHandle = 0;
+void releaseWidget() {
+	super.releaseWidget();
+
+	if (GTK.GTK4) {
+		if (parent.actionGroup != 0 && actionName != null) OS.g_action_map_remove_action(parent.actionGroup, Converter.javaStringToCString(actionName));
+	} else {
+		long accelGroup = getAccelGroup();
+		if (accelGroup != 0) removeAccelerator(accelGroup);
+
+		if (groupHandle != 0) OS.g_object_unref(groupHandle);
+		groupHandle = 0;
+		parent = null;
+	}
+
 	accelerator = 0;
-	parent = null;
+
+	disposeDefaultDisabledImage();
+}
+
+@Override
+void destroyWidget() {
+	if (GTK.GTK4) {
+		if ((style & SWT.SEPARATOR) != 0) {
+			Section aboveSection = parent.sections.get(parent.sections.indexOf(section) - 1);
+			aboveSection.sectionItems.addAll(section.sectionItems);
+
+			for (MenuItem item : section.sectionItems) {
+				OS.g_menu_insert_item(aboveSection.getSectionHandle(), aboveSection.sectionItems.indexOf(item), item.handle);
+			}
+
+			OS.g_menu_remove(parent.modelHandle, parent.sections.indexOf(section));
+
+			parent.sections.remove(section);
+		} else {
+			OS.g_menu_remove(section.getSectionHandle(), section.sectionItems.indexOf(this));
+			section.sectionItems.remove(this);
+		}
+
+		parent.items.remove(this);
+		parent = null;
+
+		if (modelHandle != 0) OS.g_object_unref(modelHandle);
+		OS.g_object_unref(handle);
+
+		releaseHandle();
+	} else {
+		super.destroyWidget();
+	}
 }
 
 void removeAccelerator (long accelGroup) {
@@ -909,7 +974,7 @@ public void setEnabled (boolean enabled) {
 	checkWidget();
 
 	if (GTK.GTK4) {
-		OS.g_simple_action_set_enabled(actionHandle, enabled);
+		if (actionHandle != 0) OS.g_simple_action_set_enabled(actionHandle, enabled);
 	} else {
 		if (GTK.gtk_widget_get_sensitive(handle) == enabled) return;
 		long accelGroup = getAccelGroup();
@@ -917,6 +982,21 @@ public void setEnabled (boolean enabled) {
 		GTK.gtk_widget_set_sensitive(handle, enabled);
 		if (accelGroup != 0) addAccelerator(accelGroup);
 	}
+
+	if (this.enabled == enabled) return;
+	this.enabled = enabled;
+
+	_setEnabledOrDisabledImage();
+}
+
+private void _setEnabledOrDisabledImage() {
+	if (!enabled) {
+		if (defaultDisableImage == null && image != null) {
+			defaultDisableImage = new Image(getDisplay(), image, SWT.IMAGE_DISABLE);
+		}
+		_setImage(defaultDisableImage);
+	}
+	if (enabled && image != null) _setImage(image);
 }
 
 /**
@@ -944,10 +1024,7 @@ public void setID (int id) {
  * <p>
  * Note: This operation is a <em>HINT</em> and is not supported on
  * platforms that do not have this concept (for example, Windows NT).
- * Furthermore, some platforms (such as GTK2), cannot display both
- * a check box and an image at the same time.  Instead, they hide
- * the image and display the check box. Some platforms (such as GTK3)
- * support images alongside check boxes.
+ * Some platforms (such as GTK3) support images alongside check boxes.
  * </p>
  *
  * @param image the image to display on the receiver (may be null)
@@ -963,9 +1040,15 @@ public void setImage (Image image) {
 	if (GTK.GTK4) return;
 
 	checkWidget();
+	if (this.image == image) return;
 	if ((style & SWT.SEPARATOR) != 0) return;
+	disposeDefaultDisabledImage();
 	super.setImage (image);
 
+	_setEnabledOrDisabledImage();
+}
+
+private void _setImage (Image image) {
 	if (image != null) {
 		ImageList imageList = parent.imageList;
 		if (imageList == null) imageList = parent.imageList = new ImageList ();
@@ -1066,8 +1149,8 @@ public void setMenu (Menu menu) {
 			OS.g_menu_item_set_submenu(handle, 0);
 		}
 
-		OS.g_menu_remove(parent.sectionModelHandle, parentMenuPosition);
-		OS.g_menu_insert_item(parent.sectionModelHandle, parentMenuPosition, handle);
+		OS.g_menu_remove(section.getSectionHandle(), section.getItemPosition(this));
+		OS.g_menu_insert_item(section.getSectionHandle(), section.getItemPosition(this), handle);
 	} else {
 		long accelGroup = getAccelGroup ();
 		if (accelGroup != 0) removeAccelerators (accelGroup);
@@ -1126,7 +1209,11 @@ public void setSelection (boolean selected) {
 	if ((style & (SWT.CHECK | SWT.RADIO)) == 0) return;
 
 	if (GTK.GTK4) {
-		OS.g_simple_action_set_state(actionHandle, OS.g_variant_new_string(Converter.javaStringToCString(selected ? "toggled" : "untoggled")));
+		if ((style & SWT.CHECK) != 0) {
+			OS.g_simple_action_set_state(actionHandle, OS.g_variant_new_boolean(selected));
+		} else {
+			OS.g_simple_action_set_state(actionHandle, OS.g_variant_new_string(Converter.javaStringToCString(selected ? "toggled" : "untoggled")));
+		}
 	} else {
 		OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, ACTIVATE);
 		GTK3.gtk_check_menu_item_set_active (handle, selected);
@@ -1197,8 +1284,8 @@ public void setText (String string) {
 					GTK.gtk_accelerator_name(maskKeysym.keysym, maskKeysym.mask)
 				);
 		}
-		OS.g_menu_remove(parent.sectionModelHandle, parentMenuPosition);
-		OS.g_menu_insert_item(parent.sectionModelHandle, parentMenuPosition, handle);
+		OS.g_menu_remove(section.getSectionHandle(), section.getItemPosition(this));
+		OS.g_menu_insert_item(section.getSectionHandle(), section.getItemPosition(this), handle);
 	} else {
 		if (labelHandle != 0 && GTK.GTK_IS_LABEL (labelHandle)) {
 			GTK.gtk_label_set_text_with_mnemonic (labelHandle, buffer);
@@ -1246,11 +1333,14 @@ public void setText (String string) {
  *
  * @since 3.104
  */
-public void setToolTipText (String toolTip) {
+public void setToolTipText(String toolTip) {
 	checkWidget();
-	if (toolTip != null && (toolTip.trim().length() == 0 || toolTip.equals (toolTipText))) return;
+	if (GTK.GTK4) return; // GTK4 does not support tooltips within menus
 
-	this.parent.getShell().setToolTipText (handle, (toolTipText = toolTip));
+	if (toolTip != null && (toolTip.trim().length() == 0 || toolTip.equals(toolTipText))) return;
+
+	toolTipText = toolTip;
+	setToolTipText(handle, toolTip);
 }
 
 void updateAccelerator (long accelGroup, boolean add) {
@@ -1280,7 +1370,7 @@ void updateAccelerator (long accelGroup, boolean add) {
 	}
 }
 
-private class MaskKeysym {
+private static class MaskKeysym {
 	int mask = 0;
 	int keysym = 0;
 }
@@ -1368,4 +1458,10 @@ long dpiChanged(long object, long arg0) {
 	return 0;
 }
 
+private void disposeDefaultDisabledImage() {
+	if (defaultDisableImage != null) {
+		defaultDisableImage.dispose();
+		defaultDisableImage = null;
+	}
+}
 }
