@@ -140,6 +140,22 @@ public void addSelectionListener (SelectionListener listener) {
 }
 
 void configure () {
+	/*
+	 * Bug in GTK4: A lot of the functions that SWT uses to configure tooltips
+	 * in this function were removed from GTK4. Due to this, whenever configure()
+	 * gets called it causes a crash. The replacement functions for
+	 * gtk_widget_shape_combine_region has not been found. So for now, allow this
+	 * function to be called without causing the program to crash.
+	 *
+	 * TODO: Find the replacement for all GTK3 functions called here
+	 *
+	 * See Bug 577600
+	 */
+	if(GTK.GTK4) {
+		System.err.println("SWT Error: ToolTip.java: ToolTip with style SWT.BALLOON not supported on GTK 4.");
+		return;
+	}
+
 	GTK.gtk_widget_realize (handle);
 	/*
 	 * Feature in GTK: using gdk_screen_get_monitor_at_window() does not accurately get the correct monitor on the machine.
@@ -149,25 +165,18 @@ void configure () {
 	Point point = getLocation();
 	boolean multipleMonitors;
 	GdkRectangle dest = new GdkRectangle ();
-	if (GTK.GTK_VERSION >= OS.VERSION(3, 22, 0)) {
-		long display = GDK.gdk_display_get_default();
+	long displayHandle = GDK.gdk_display_get_default();
 
-		if (GTK.GTK4) {
-			long monitorList = GDK.gdk_display_get_monitors(display);
-			multipleMonitors = OS.g_list_model_get_n_items(monitorList) > 1;
-		} else {
-			multipleMonitors = GDK.gdk_display_get_n_monitors(display) > 1;
-		}
-
-
-		long monitor = GDK.gdk_display_get_monitor_at_point(display, point.x, point.y);
-		GDK.gdk_monitor_get_geometry (monitor, dest);
+	if (GTK.GTK4) {
+		long monitorList = GDK.gdk_display_get_monitors(displayHandle);
+		multipleMonitors = OS.g_list_model_get_n_items(monitorList) > 1;
 	} else {
-		long screen = GDK.gdk_screen_get_default ();
-		multipleMonitors = GDK.gdk_screen_get_n_monitors(screen) > 1;
-		int monitorNumber = GDK.gdk_screen_get_monitor_at_point(screen, point.x, point.y);
-		GDK.gdk_screen_get_monitor_geometry(screen, monitorNumber, dest);
+		multipleMonitors = GDK.gdk_display_get_n_monitors(displayHandle) > 1;
 	}
+
+
+	long monitor = GDK.gdk_display_get_monitor_at_point(displayHandle, point.x, point.y);
+	GDK.gdk_monitor_get_geometry (monitor, dest);
 	point = getSize (dest.width / 4);
 	int w = point.x;
 	int h = point.y;
@@ -371,7 +380,7 @@ Point getLocation () {
 	if (item != null) {
 		long itemHandle = item.handle;
 		GdkRectangle area = new GdkRectangle ();
-		GTK.gtk_status_icon_get_geometry (itemHandle, 0, area, 0);
+		GTK3.gtk_status_icon_get_geometry (itemHandle, 0, area, 0);
 		x = area.x + area.width / 2;
 		y = area.y + area.height / 2;
 	}
@@ -579,15 +588,9 @@ long gtk_size_allocate (long widget, long allocation) {
 	int y = point.y;
 	GTK.gtk_widget_realize (widget);
 	GdkRectangle dest = new GdkRectangle ();
-	if (GTK.GTK_VERSION >= OS.VERSION(3, 22, 0)) {
-		long display = GDK.gdk_display_get_default();
-		long monitor = GDK.gdk_display_get_monitor_at_point(display, x, y);
-		GDK.gdk_monitor_get_geometry(monitor, dest);
-	} else {
-		long screen = GDK.gdk_screen_get_default ();
-		int monitorNumber = GDK.gdk_screen_get_monitor_at_point(screen, point.x, point.y);
-		GDK.gdk_screen_get_monitor_geometry (screen, monitorNumber, dest);
-	}
+	long display = GDK.gdk_display_get_default();
+	long monitor = GDK.gdk_display_get_monitor_at_point(display, x, y);
+	GDK.gdk_monitor_get_geometry(monitor, dest);
 	GtkAllocation widgetAllocation = new GtkAllocation ();
 	GTK.gtk_widget_get_allocation (widget, widgetAllocation);
 	int w = widgetAllocation.width;
