@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2018 IBM Corporation and others.
+ * Copyright (c) 2000, 2022 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -111,16 +111,6 @@ public final class Image extends Resource implements Drawable {
 	 */
 	GC memGC;
 
-	/**
-	 * the alpha data for the image
-	 */
-	byte[] alphaData;
-
-	/**
-	 * the global alpha value to be used for every pixel
-	 */
-	int alpha = -1;
-
 	/**
 	 * ImageFileNameProvider to provide file names at various Zoom levels
 	 */
@@ -283,11 +273,6 @@ public Image(Device device, Image srcImage, int flag) {
 					device.internal_dispose_GC(hDC, null);
 
 					transparentPixel = srcImage.transparentPixel;
-					alpha = srcImage.alpha;
-					if (srcImage.alphaData != null) {
-						alphaData = new byte[srcImage.alphaData.length];
-						System.arraycopy(srcImage.alphaData, 0, alphaData, 0, alphaData.length);
-					}
 					break;
 				case SWT.ICON:
 					handle = OS.CopyImage(srcImage.handle, OS.IMAGE_ICON, rect.width, rect.height, 0);
@@ -820,7 +805,7 @@ void initNative(String filename) {
 	* Windows 7 when the image has a position offset in the first frame.
 	* The fix is to not use GDI+ image loading in this case.
 	*/
-	if (OS.WIN32_VERSION >= OS.VERSION(6, 1) && filename.toLowerCase().endsWith(".gif")) gdip = false;
+	if (filename.toLowerCase().endsWith(".gif")) gdip = false;
 	if (gdip) {
 		int length = filename.length();
 		char[] chars = new char[length+1];
@@ -977,15 +962,26 @@ void initNative(String filename) {
 long [] createGdipImage() {
 	switch (type) {
 		case SWT.BITMAP: {
-			if (alpha != -1 || alphaData != null || transparentPixel != -1) {
-				BITMAP bm = new BITMAP();
-				OS.GetObject(handle, BITMAP.sizeof, bm);
+			BITMAP bm = new BITMAP();
+			OS.GetObject(handle, BITMAP.sizeof, bm);
+			int depth = bm.bmPlanes * bm.bmBitsPixel;
+			boolean isDib = bm.bmBits != 0;
+			boolean hasAlpha = isDib && depth == 32;
+			if (hasAlpha || transparentPixel != -1) {
 				int imgWidth = bm.bmWidth;
 				int imgHeight = bm.bmHeight;
-				long hDC = device.internal_new_GC(null);
-				long srcHdc = OS.CreateCompatibleDC(hDC);
+				long srcHdc;
+				long memHdc;
+				{
+					long hDC = device.internal_new_GC(null);
+					if (hDC == 0) SWT.error(SWT.ERROR_NO_HANDLES);
+					srcHdc = OS.CreateCompatibleDC(hDC);
+					memHdc = OS.CreateCompatibleDC(hDC);
+					device.internal_dispose_GC(hDC, null);
+				}
+				if (srcHdc == 0) SWT.error(SWT.ERROR_NO_HANDLES);
+				if (memHdc == 0) SWT.error(SWT.ERROR_NO_HANDLES);
 				long oldSrcBitmap = OS.SelectObject(srcHdc, handle);
-				long memHdc = OS.CreateCompatibleDC(hDC);
 				long memDib = createDIB(imgWidth, imgHeight, 32);
 				if (memDib == 0) SWT.error(SWT.ERROR_NO_HANDLES);
 				long oldMemBitmap = OS.SelectObject(memHdc, memDib);
@@ -993,8 +989,14 @@ long [] createGdipImage() {
 				OS.GetObject(memDib, BITMAP.sizeof, dibBM);
 				int sizeInBytes = dibBM.bmWidthBytes * dibBM.bmHeight;
 				OS.BitBlt(memHdc, 0, 0, imgWidth, imgHeight, srcHdc, 0, 0, OS.SRCCOPY);
+				long hHeap = OS.GetProcessHeap();
+				long pixels = OS.HeapAlloc(hHeap, OS.HEAP_ZERO_MEMORY, sizeInBytes);
+				if (pixels == 0) SWT.error(SWT.ERROR_NO_HANDLES);
 				byte red = 0, green = 0, blue = 0;
-				if (transparentPixel != -1) {
+				if (hasAlpha) {
+					OS.MoveMemory(pixels, bm.bmBits, sizeInBytes);
+				}
+				else {
 					if (bm.bmBitsPixel <= 8)  {
 						byte[] color = new byte[4];
 						OS.GetDIBColorTable(srcHdc, transparentPixel, 1, color);
@@ -1029,30 +1031,8 @@ long [] createGdipImage() {
 								break;
 						}
 					}
-				}
-				OS.SelectObject(srcHdc, oldSrcBitmap);
-				OS.SelectObject(memHdc, oldMemBitmap);
-				OS.DeleteObject(srcHdc);
-				OS.DeleteObject(memHdc);
-				byte[] srcData = new byte[sizeInBytes];
-				OS.MoveMemory(srcData, dibBM.bmBits, sizeInBytes);
-				OS.DeleteObject(memDib);
-				device.internal_dispose_GC(hDC, null);
-				if (alpha != -1) {
-					for (int y = 0, dp = 0; y < imgHeight; ++y) {
-						for (int x = 0; x < imgWidth; ++x) {
-							srcData[dp + 3] = (byte)alpha;
-							dp += 4;
-						}
-					}
-				} else if (alphaData != null) {
-					for (int y = 0, dp = 0, ap = 0; y < imgHeight; ++y) {
-						for (int x = 0; x < imgWidth; ++x) {
-							srcData[dp + 3] = alphaData[ap++];
-							dp += 4;
-						}
-					}
-				} else if (transparentPixel != -1) {
+					byte[] srcData = new byte[sizeInBytes];
+					OS.MoveMemory(srcData, dibBM.bmBits, sizeInBytes);
 					for (int y = 0, dp = 0; y < imgHeight; ++y) {
 						for (int x = 0; x < imgWidth; ++x) {
 							if (srcData[dp] == blue && srcData[dp + 1] == green && srcData[dp + 2] == red) {
@@ -1063,12 +1043,15 @@ long [] createGdipImage() {
 							dp += 4;
 						}
 					}
+					OS.MoveMemory(pixels, srcData, sizeInBytes);
 				}
-				long hHeap = OS.GetProcessHeap();
-				long pixels = OS.HeapAlloc(hHeap, OS.HEAP_ZERO_MEMORY, srcData.length);
-				if (pixels == 0) SWT.error(SWT.ERROR_NO_HANDLES);
-				OS.MoveMemory(pixels, srcData, sizeInBytes);
-				return new long []{Gdip.Bitmap_new(imgWidth, imgHeight, dibBM.bmWidthBytes, Gdip.PixelFormat32bppARGB, pixels), pixels};
+				OS.SelectObject(srcHdc, oldSrcBitmap);
+				OS.SelectObject(memHdc, oldMemBitmap);
+				OS.DeleteObject(srcHdc);
+				OS.DeleteObject(memHdc);
+				OS.DeleteObject(memDib);
+				int pixelFormat = hasAlpha ? Gdip.PixelFormat32bppPARGB : Gdip.PixelFormat32bppARGB;
+				return new long []{Gdip.Bitmap_new(imgWidth, imgHeight, dibBM.bmWidthBytes, pixelFormat, pixels), pixels};
 			}
 			return new long []{Gdip.Bitmap_new(handle, 0), 0};
 		}
@@ -1190,7 +1173,7 @@ public boolean equals (Object object) {
  * color of the widget to paint the transparent pixels of the image.
  * Use this method to check which color will be used in these cases
  * in place of transparency. This value may be set with setBackground().
- * <p>
+ * </p>
  *
  * @return the background color of the image, or null if there is no transparency in the image
  *
@@ -1633,10 +1616,32 @@ public ImageData getImageDataAtCurrentZoom() {
 			/* Construct and return the ImageData */
 			ImageData imageData = new ImageData(width, height, depth, palette, 4, data);
 			imageData.transparentPixel = this.transparentPixel;
-			imageData.alpha = alpha;
-			if (alpha == -1 && alphaData != null) {
-				imageData.alphaData = new byte[alphaData.length];
-				System.arraycopy(alphaData, 0, imageData.alphaData, 0, alphaData.length);
+			if (depth == 32) {
+				byte straightData[] = new byte[imageSize];
+				byte alphaData[] = new byte[width * height];
+				boolean validAlpha = isDib;
+				for (int ap = 0, dp = 0; validAlpha && ap < alphaData.length; ap++, dp += 4) {
+					int b = data[dp    ] & 0xFF;
+					int g = data[dp + 1] & 0xFF;
+					int r = data[dp + 2] & 0xFF;
+					int a = data[dp + 3] & 0xFF;
+					alphaData[ap] = (byte) a;
+					validAlpha = validAlpha && b <= a && g <= a && r <= a;
+					if (a != 0) {
+						straightData[dp    ] = (byte) (((b * 0xFF) + a / 2) / a);
+						straightData[dp + 1] = (byte) (((g * 0xFF) + a / 2) / a);
+						straightData[dp + 2] = (byte) (((r * 0xFF) + a / 2) / a);
+					}
+				}
+				if (validAlpha) {
+					imageData.data = straightData;
+					imageData.alphaData = alphaData;
+				}
+				else {
+					for (int dp = 3; dp < imageSize; dp += 4) {
+						data[dp] = (byte) 0xFF;
+					}
+				}
 			}
 			return imageData;
 		}
@@ -1684,6 +1689,7 @@ void init(int width, int height) {
 		int planes = OS.GetDeviceCaps(hDC, OS.PLANES);
 		int depth = bits * planes;
 		if (depth < 16) depth = 16;
+		if (depth > 24) depth = 24;
 		handle = createDIB(width, height, depth);
 	}
 	if (handle != 0) {
@@ -1713,22 +1719,89 @@ static long createDIB(int width, int height, int depth) {
 	return OS.CreateDIBSection(0, bmi, OS.DIB_RGB_COLORS, pBits, 0, 0);
 }
 
+static ImageData indexToIndex(ImageData src, int newDepth) {
+	ImageData img = new ImageData(src.width, src.height, newDepth, src.palette);
+
+	ImageData.blit(
+		src.data, src.depth, src.bytesPerLine, src.getByteOrder(), src.width, src.height,
+		img.data, img.depth, img.bytesPerLine, src.getByteOrder(), img.width, img.height,
+		false, false);
+
+	img.transparentPixel = src.transparentPixel;
+	img.maskPad   = src.maskPad;
+	img.maskData  = src.maskData;
+	img.alpha     = src.alpha;
+	img.alphaData = src.alphaData;
+
+	return img;
+}
+
+static ImageData indexToDirect(ImageData src, int newDepth, PaletteData newPalette, int newByteOrder) {
+	ImageData img = new ImageData(src.width, src.height, newDepth, newPalette);
+
+	RGB[] rgbs = src.palette.getRGBs();
+	byte[] srcReds   = new byte[rgbs.length];
+	byte[] srcGreens = new byte[rgbs.length];
+	byte[] srcBlues  = new byte[rgbs.length];
+	for (int j = 0; j < rgbs.length; j++) {
+		RGB rgb = rgbs[j];
+		if (rgb == null) continue;
+		srcReds[j] = (byte)rgb.red;
+		srcGreens[j] = (byte)rgb.green;
+		srcBlues[j] = (byte)rgb.blue;
+	}
+
+	ImageData.blit(
+		src.width, src.height,
+		src.data, src.depth, src.bytesPerLine, src.getByteOrder(), srcReds, srcGreens, srcBlues,
+		img.data, img.depth, img.bytesPerLine,       newByteOrder, newPalette.redMask, newPalette.greenMask, newPalette.blueMask);
+
+	if (src.transparentPixel != -1) {
+		img.transparentPixel = newPalette.getPixel(src.palette.getRGB(src.transparentPixel));
+	}
+
+	img.maskPad   = src.maskPad;
+	img.maskData  = src.maskData;
+	img.alpha     = src.alpha;
+	img.alphaData = src.alphaData;
+
+	return img;
+}
+
+static ImageData directToDirect(ImageData src, int newDepth, PaletteData newPalette, int newByteOrder) {
+	ImageData img = new ImageData(src.width, src.height, newDepth, newPalette);
+
+	ImageData.blit(
+		src.data, src.depth, src.bytesPerLine, src.getByteOrder(), src.width, src.height, src.palette.redMask, src.palette.greenMask, src.palette.blueMask,
+		img.data, img.depth, img.bytesPerLine,       newByteOrder, img.width, img.height, img.palette.redMask, img.palette.greenMask, img.palette.blueMask,
+		false, false);
+
+	if (src.transparentPixel != -1) {
+		img.transparentPixel = img.palette.getPixel(src.palette.getRGB(src.transparentPixel));
+	}
+
+	img.maskPad   = src.maskPad;
+	img.maskData  = src.maskData;
+	img.alpha     = src.alpha;
+	img.alphaData = src.alphaData;
+
+	return img;
+}
+
 static long [] init(Device device, Image image, ImageData i) {
 	/* Windows does not support 2-bit images. Convert to 4-bit image. */
 	if (i.depth == 2) {
-		ImageData img = new ImageData(i.width, i.height, 4, i.palette);
-		ImageData.blit(ImageData.BLIT_SRC,
-			i.data, i.depth, i.bytesPerLine, i.getByteOrder(), 0, 0, i.width, i.height, null, null, null,
-			ImageData.ALPHA_OPAQUE, null, 0, 0, 0,
-			img.data, img.depth, img.bytesPerLine, i.getByteOrder(), 0, 0, img.width, img.height, null, null, null,
-			false, false);
-		img.transparentPixel = i.transparentPixel;
-		img.maskPad = i.maskPad;
-		img.maskData = i.maskData;
-		img.alpha = i.alpha;
-		img.alphaData = i.alphaData;
-		i = img;
+		i = indexToIndex(i, 4);
 	}
+
+	/* Windows does not support 16-bit palette images. Convert to RGB. */
+	if ((i.depth == 16) && !i.palette.isDirect) {
+		PaletteData newPalette = new PaletteData(0xFF, 0xFF00, 0xFF0000);
+		i = indexToDirect(i, 24, newPalette, ImageData.MSB_FIRST);
+	}
+
+	boolean hasAlpha = i.alpha != -1 || i.alphaData != null;
+
 	/*
 	 * Windows supports 16-bit mask of (0x7C00, 0x3E0, 0x1F),
 	 * 24-bit mask of (0xFF0000, 0xFF00, 0xFF) and 32-bit mask
@@ -1745,48 +1818,105 @@ static long [] init(Device device, Image image, ImageData i) {
 		int newOrder = ImageData.MSB_FIRST;
 		PaletteData newPalette = null;
 
-		switch (i.depth) {
-			case 8:
-				newDepth = 16;
-				newOrder = ImageData.LSB_FIRST;
-				newPalette = new PaletteData(0x7C00, 0x3E0, 0x1F);
-				break;
-			case 16:
-				newOrder = ImageData.LSB_FIRST;
-				if (!(redMask == 0x7C00 && greenMask == 0x3E0 && blueMask == 0x1F)) {
-					newPalette = new PaletteData(0x7C00, 0x3E0, 0x1F);
-				}
-				break;
-			case 24:
-				if (!(redMask == 0xFF && greenMask == 0xFF00 && blueMask == 0xFF0000)) {
-					newPalette = new PaletteData(0xFF, 0xFF00, 0xFF0000);
-				}
-				break;
-			case 32:
-				if (!(redMask == 0xFF00 && greenMask == 0xFF0000 && blueMask == 0xFF000000)) {
-					newPalette = new PaletteData(0xFF00, 0xFF0000, 0xFF000000);
-				}
-				break;
-			default:
-				SWT.error(SWT.ERROR_UNSUPPORTED_DEPTH);
+		if (hasAlpha) {
+			newDepth = 32;
+			newPalette = new PaletteData(0xFF00, 0xFF0000, 0xFF000000);
+		}
+		else {
+			switch (i.depth) {
+				case 8:
+					/*
+					 * Bug 566545. Usually each color mask selects a different part of the pixel
+					 * value to encode the according color. In this common case it is rather trivial
+					 * to convert an 8-bit direct color image to the Windows supported 16-bit image.
+					 * However there is no enforcement for the color masks to be disjunct. For
+					 * example an 8-bit image where all color masks select the same 8-bit of pixel
+					 * value (mask = 0xFF and shift = 0 for all colors) results in a very efficient
+					 * 8-bit gray-scale image without the need of defining a color table.
+					 *
+					 * That's why we need to calculate the actual required depth if all colors are
+					 * stored non-overlapping which might require 24-bit instead of the usual
+					 * expected 16-bit.
+					 */
+					int minDepth = ImageData.getChannelWidth(redMask, palette.redShift)
+							+ ImageData.getChannelWidth(greenMask, palette.greenShift)
+							+ ImageData.getChannelWidth(blueMask, palette.blueShift);
+					if (minDepth <= 16) {
+						newDepth = 16;
+						newOrder = ImageData.LSB_FIRST;
+						newPalette = new PaletteData(0x7C00, 0x3E0, 0x1F);
+					} else {
+						newDepth = 24;
+						newPalette = new PaletteData(0xFF, 0xFF00, 0xFF0000);
+					}
+					break;
+				case 16:
+					newOrder = ImageData.LSB_FIRST;
+					if (!(redMask == 0x7C00 && greenMask == 0x3E0 && blueMask == 0x1F)) {
+						newPalette = new PaletteData(0x7C00, 0x3E0, 0x1F);
+					}
+					break;
+				case 24:
+					if (!(redMask == 0xFF && greenMask == 0xFF00 && blueMask == 0xFF0000)) {
+						newPalette = new PaletteData(0xFF, 0xFF00, 0xFF0000);
+					}
+					break;
+				case 32:
+					if (i.getTransparencyType() != SWT.TRANSPARENCY_MASK) {
+						newDepth = 24;
+						newPalette = new PaletteData(0xFF, 0xFF00, 0xFF0000);
+					}
+					else if (!(redMask == 0xFF00 && greenMask == 0xFF0000 && blueMask == 0xFF000000)) {
+						newPalette = new PaletteData(0xFF00, 0xFF0000, 0xFF000000);
+					}
+					break;
+				default:
+					SWT.error(SWT.ERROR_UNSUPPORTED_DEPTH);
+			}
 		}
 		if (newPalette != null) {
-			ImageData img = new ImageData(i.width, i.height, newDepth, newPalette);
-			ImageData.blit(ImageData.BLIT_SRC,
-					i.data, i.depth, i.bytesPerLine, i.getByteOrder(), 0, 0, i.width, i.height, redMask, greenMask, blueMask,
-					ImageData.ALPHA_OPAQUE, null, 0, 0, 0,
-					img.data, img.depth, img.bytesPerLine, newOrder, 0, 0, img.width, img.height, newPalette.redMask, newPalette.greenMask, newPalette.blueMask,
-					false, false);
-			if (i.transparentPixel != -1) {
-				img.transparentPixel = newPalette.getPixel(palette.getRGB(i.transparentPixel));
-			}
-			img.maskPad = i.maskPad;
-			img.maskData = i.maskData;
-			img.alpha = i.alpha;
-			img.alphaData = i.alphaData;
-			i = img;
+			i = directToDirect(i, newDepth, newPalette, newOrder);
 		}
 	}
+	else if (hasAlpha) {
+		PaletteData newPalette = new PaletteData(0xFF00, 0xFF0000, 0xFF000000);
+		i = indexToDirect(i, 32, newPalette, ImageData.MSB_FIRST);
+	}
+	if (i.alpha != -1) {
+		int alpha = i.alpha & 0xFF;
+		byte[] data = i.data;
+		for (int dp = 0; dp < i.data.length; dp += 4) {
+			/* pre-multiplied alpha */
+			int r = ((data[dp    ] & 0xFF) * alpha) + 128;
+			r = (r + (r >> 8)) >> 8;
+			int g = ((data[dp + 1] & 0xFF) * alpha) + 128;
+			g = (g + (g >> 8)) >> 8;
+			int b = ((data[dp + 2] & 0xFF) * alpha) + 128;
+			b = (b + (b >> 8)) >> 8;
+			data[dp    ] = (byte) b;
+			data[dp + 1] = (byte) g;
+			data[dp + 2] = (byte) r;
+			data[dp + 3] = (byte) alpha;
+		}
+	}
+	else if (i.alphaData != null) {
+		byte[] data = i.data;
+		for (int ap = 0, dp = 0; dp < i.data.length; ap++, dp += 4) {
+			/* pre-multiplied alpha */
+			int a = i.alphaData[ap] & 0xFF;
+			int r = ((data[dp    ] & 0xFF) * a) + 128;
+			r = (r + (r >> 8)) >> 8;
+			int g = ((data[dp + 1] & 0xFF) * a) + 128;
+			g = (g + (g >> 8)) >> 8;
+			int b = ((data[dp + 2] & 0xFF) * a) + 128;
+			b = (b + (b >> 8)) >> 8;
+			data[dp    ] = (byte) r;
+			data[dp + 1] = (byte) g;
+			data[dp + 2] = (byte) b;
+			data[dp + 3] = (byte) a;
+		}
+	}
+
 	/* Construct bitmap info header by hand */
 	RGB[] rgbs = i.palette.getRGBs();
 	BITMAPINFOHEADER bmiHeader = new BITMAPINFOHEADER();
@@ -1873,14 +2003,6 @@ static long [] init(Device device, Image image, ImageData i) {
 			image.handle = hDib;
 			image.type = SWT.BITMAP;
 			image.transparentPixel = i.transparentPixel;
-			if (image.transparentPixel == -1) {
-				image.alpha = i.alpha;
-				if (i.alpha == -1 && i.alphaData != null) {
-					int length = i.alphaData.length;
-					image.alphaData = new byte[length];
-					System.arraycopy(i.alphaData, 0, image.alphaData, 0, length);
-				}
-			}
 		}
 	}
 	return result;
