@@ -49,7 +49,7 @@ public class Transform extends Resource {
 	 *
 	 * @noreference This field is not intended to be referenced by clients.
 	 */
-	public long /*int*/ handle;
+	public long handle;
 
 /**
  * Constructs a new identity Transform.
@@ -147,7 +147,8 @@ public Transform(Device device, float[] elements) {
 public Transform (Device device, float m11, float m12, float m21, float m22, float dx, float dy) {
 	super(device);
 	this.device.checkGDIP();
-	handle = Gdip.Matrix_new(m11, m12, m21, m22, DPIUtil.autoScaleUp(dx), DPIUtil.autoScaleUp(dy));
+	handle = Gdip.Matrix_new(m11, m12, m21, m22, 
+	        DPIUtil.autoScaleUp(this.device, dx), DPIUtil.autoScaleUp(this.device, dy));
 	if (handle == 0) SWT.error(SWT.ERROR_NO_HANDLES);
 	init();
 }
@@ -183,8 +184,9 @@ public void getElements(float[] elements) {
 	if (elements == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
 	if (elements.length < 6) SWT.error(SWT.ERROR_INVALID_ARGUMENT);
 	Gdip.Matrix_GetElements(handle, elements);
-	elements[4] = DPIUtil.autoScaleDown(elements[4]);
-	elements[5] = DPIUtil.autoScaleDown(elements[5]);
+	Drawable drawable = getDevice();
+	elements[4] = DPIUtil.autoScaleDown(drawable, elements[4]);
+	elements[5] = DPIUtil.autoScaleDown(drawable, elements[5]);
 }
 
 /**
@@ -315,7 +317,9 @@ public void scale(float scaleX, float scaleY) {
  */
 public void setElements(float m11, float m12, float m21, float m22, float dx, float dy) {
 	if (isDisposed()) SWT.error(SWT.ERROR_GRAPHIC_DISPOSED);
-	Gdip.Matrix_SetElements(handle, m11, m12, m21, m22, DPIUtil.autoScaleUp(dx), DPIUtil.autoScaleUp(dy));
+	Drawable drawable = getDevice();
+	Gdip.Matrix_SetElements(handle, m11, m12, m21, m22, 
+	        DPIUtil.autoScaleUp(drawable, dx), DPIUtil.autoScaleUp(drawable, dy));
 }
 
 /**
@@ -354,12 +358,13 @@ public void transform(float[] pointArray) {
 	if (isDisposed()) SWT.error(SWT.ERROR_GRAPHIC_DISPOSED);
 	if (pointArray == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
 	int length = pointArray.length;
+	Drawable drawable = getDevice();
 	for (int i = 0; i < length; i++) {
-		pointArray[i] = DPIUtil.autoScaleUp(pointArray[i]);
+		pointArray[i] = DPIUtil.autoScaleUp(drawable, pointArray[i]);
 	}
 	Gdip.Matrix_TransformPoints(handle, pointArray, length / 2);
 	for (int i = 0; i < length; i++) {
-		pointArray[i] = DPIUtil.autoScaleDown(pointArray[i]);
+		pointArray[i] = DPIUtil.autoScaleDown(drawable, pointArray[i]);
 	}
 }
 
@@ -376,7 +381,8 @@ public void transform(float[] pointArray) {
  */
 public void translate(float offsetX, float offsetY) {
 	if (isDisposed()) SWT.error(SWT.ERROR_GRAPHIC_DISPOSED);
-	Gdip.Matrix_Translate(handle, DPIUtil.autoScaleUp(offsetX), DPIUtil.autoScaleUp(offsetY), Gdip.MatrixOrderPrepend);
+	Drawable drawable = getDevice();
+	Gdip.Matrix_Translate(handle, DPIUtil.autoScaleUp(drawable, offsetX), DPIUtil.autoScaleUp(drawable, offsetY), Gdip.MatrixOrderPrepend);
 }
 
 /**
