@@ -916,12 +916,19 @@ void releaseParent () {
 
 @Override
 void releaseChildren (boolean destroy) {
-	for (Control child : _getChildren ()) {
-		if (child != null && !child.isDisposed ()) {
-			child.release (false);
+	try (ExceptionStash exceptions = new ExceptionStash ()) {
+		for (Control child : _getChildren ()) {
+			if (child == null || child.isDisposed ())
+				continue;
+
+			try {
+				child.release (false);
+			} catch (Error | RuntimeException ex) {
+				exceptions.stash (ex);
+			}
 		}
+		super.releaseChildren (destroy);
 	}
-	super.releaseChildren (destroy);
 }
 
 @Override
@@ -1954,4 +1961,9 @@ LRESULT wmNotify (NMHDR hdr, long wParam, long lParam) {
 	return super.wmNotify (hdr, wParam, lParam);
 }
 
+@Override
+public String toString() {
+	return super.toString() + " [layout=" + layout + "]";
+}
+
 }
