@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2016 IBM Corporation and others.
+ * Copyright (c) 2000, 2021 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -575,6 +575,24 @@ void createBalloonTipHandle () {
 	OS.SetWindowLongPtr (balloonTipHandle, OS.GWLP_WNDPROC, display.windowProc);
 }
 
+void setTitleColoring() {
+	int attributeID = 0;
+	if (OS.WIN32_BUILD >= OS.WIN32_BUILD_WIN10_2004) {
+		// Documented since build 20348, but was already present since build 19041
+		final int DWMWA_USE_IMMERSIVE_DARK_MODE = 20;
+		attributeID = DWMWA_USE_IMMERSIVE_DARK_MODE;
+	} else if (OS.WIN32_BUILD >= OS.WIN32_BUILD_WIN10_1809) {
+		// Undocumented value
+		attributeID = 19;
+	} else {
+		// Not supported
+		return;
+	}
+
+	int[] value = new int[] {1};
+	OS.DwmSetWindowAttribute (handle, attributeID, value, 4);
+}
+
 @Override
 void createHandle () {
 	boolean embedded = handle != 0 && (state & FOREIGN_HANDLE) == 0;
@@ -607,6 +625,10 @@ void createHandle () {
 //	if ((style & SWT.ON_TOP) != 0)  display.lockActiveWindow = false;
 
 	if (!embedded) {
+		if (display.useShellTitleColoring) {
+			setTitleColoring();
+		}
+
 		int bits = OS.GetWindowLong (handle, OS.GWL_STYLE);
 		bits &= ~(OS.WS_OVERLAPPED | OS.WS_CAPTION);
 		bits |= OS.WS_POPUP;
@@ -946,7 +968,6 @@ public boolean getEnabled () {
 /**
  * Returns <code>true</code> if the receiver is currently
  * in fullscreen state, and false otherwise.
- * <p>
  *
  * @return the fullscreen state
  *
@@ -1147,7 +1168,7 @@ public Shell getShell () {
 /**
  * Returns an array containing all shells which are
  * descendants of the receiver.
- * <p>
+ *
  * @return the dialog shells
  *
  * @exception SWTException <ul>
@@ -1185,7 +1206,6 @@ public Shell [] getShells () {
  * trim. This will return <code>null</code> if the platform does not support tool bars that
  * are not part of the content area of the shell, or if the Shell's style does not support
  * having a tool bar.
- * <p>
  *
  * @return a ToolBar object representing the Shell's tool bar, or <code>null</code>.
  *
@@ -1907,6 +1927,7 @@ void setToolTipText (long hwnd, String text) {
 	lpti.hwnd = handle;
 	lpti.uId = hwnd;
 	long hwndToolTip = toolTipHandle ();
+	maybeEnableDarkSystemTheme(hwndToolTip);
 	if (text == null) {
 		OS.SendMessage (hwndToolTip, OS.TTM_DELTOOL, 0, lpti);
 	} else if (OS.SendMessage (hwndToolTip, OS.TTM_GETTOOLINFO, 0, lpti) == 0) {
@@ -2316,25 +2337,6 @@ LRESULT WM_DESTROY (long wParam, long lParam) {
 	return result;
 }
 
-@Override
-LRESULT WM_ERASEBKGND (long wParam, long lParam) {
-	LRESULT result = super.WM_ERASEBKGND (wParam, lParam);
-	if (result != null) return result;
-	/*
-	* Feature in Windows.  When a shell is resized by dragging
-	* the resize handles, Windows temporarily fills in black
-	* rectangles where the new contents of the shell should
-	* draw.  The fix is to always draw the background of shells.
-	*
-	* NOTE: This only happens on Vista.
-	*/
-	if (OS.WIN32_VERSION == OS.VERSION (6, 0)) {
-		drawBackground (wParam);
-		return LRESULT.ONE;
-	}
-	return result;
-}
-
 @Override
 LRESULT WM_ENTERIDLE (long wParam, long lParam) {
 	LRESULT result = super.WM_ENTERIDLE (wParam, lParam);
