@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2009, 2017 IBM Corporation and others.
+ * Copyright (c) 2009, 2020 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -26,7 +26,7 @@ import org.eclipse.swt.widgets.*;
 
 public class Snippet327 {
 
-	final static String PREAMBLE = "SNIPPET327LINK";
+	final static String PREAMBLE = "https://git.eclipse.org/c/platform/eclipse.platform.swt.git/log/?ofs=";
 
 public static void main(String[] args) {
 	Display display = new Display();
