@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2018 Red Hat and others.
+ * Copyright (c) 2018, 2019 Red Hat and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -21,29 +21,29 @@ import org.eclipse.swt.widgets.Shell;
 
 public class Bug336238_ShellBoundsTest {
 
-  static int cycles = 5;
- 	
+	static int cycles = 5;
+
 //  @Test
-  public void testSetBounds() {
-		
-    int x;
-    int y;
-    int width = 100;
-    int height = 100;
-		
-    for (int i=0; i < cycles; i++) {
-			
-      x = (new Double(Math.random()*1000)).intValue();
-      y = (new Double(Math.random()*1000)).intValue();
-			
-      Shell testShell = new Shell();
-      testShell.open();
-			
-      testShell.setBounds(x, y, width, height);
-			
+	public void testSetBounds() {
+
+		int x;
+		int y;
+		int width = 100;
+		int height = 100;
+
+		for (int i=0; i < cycles; i++) {
+
+		x = (Double.valueOf(Math.random()*1000)).intValue();
+		y = (Double.valueOf(Math.random()*1000)).intValue();
+
+			Shell testShell = new Shell();
+			testShell.open();
+
+			testShell.setBounds(x, y, width, height);
+
 //      assertEquals(x, testShell.getLocation().x);
 //      assertEquals(y, testShell.getLocation().y);
-      testShell.close();
-    }		
-  }	
+			testShell.close();
+		}
+	}
 }
\ No newline at end of file
