@@ -14,7 +14,8 @@
  */
 package org.eclipse.swt.tests.win32;
 
-import org.eclipse.swt.widgets.Test_org_eclipse_swt_widgets_Display;
+import org.eclipse.swt.tests.win32.widgets.TestTreeColumn;
+import org.eclipse.swt.tests.win32.widgets.Test_org_eclipse_swt_widgets_Display;
 import org.junit.runner.JUnitCore;
 import org.junit.runner.RunWith;
 import org.junit.runners.Suite;
@@ -24,6 +25,7 @@ import org.junit.runners.Suite;
 @Suite.SuiteClasses({
 	Test_org_eclipse_swt_dnd_DND.class,
 	Test_org_eclipse_swt_widgets_Display.class,
+	TestTreeColumn.class
 })
 
 public class AllWin32Tests {
