@@ -15,6 +15,7 @@ package org.eclipse.swt.tests.junit;
 
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNotEquals;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
@@ -180,7 +181,6 @@ public void test_setLinkForegroundLorg_eclipse_swt_graphics_Color() {
 	link.setLinkForeground(color);
 	assertEquals(color, link.getLinkForeground());
 	link.setLinkForeground(null);
-	assertFalse(link.getForeground().equals(color));
-	color.dispose();
+	assertNotEquals(color, link.getForeground());
 }
 }
